@@ -86,7 +86,7 @@ bonobo_client_site_destroy (GtkObject *object)
 	while (client_site->view_frames) {
 		BonoboViewFrame *view_frame = BONOBO_VIEW_FRAME (client_site->view_frames->data);
 
-		bonobo_object_destroy (BONOBO_OBJECT (view_frame));
+		bonobo_object_unref (BONOBO_OBJECT (view_frame));
 	}
 
 	/*
@@ -95,11 +95,16 @@ bonobo_client_site_destroy (GtkObject *object)
 	while (client_site->canvas_items) {
 		BonoboCanvasItem *item = BONOBO_CANVAS_ITEM (client_site->canvas_items->data);
 
-		bonobo_object_destroy (BONOBO_OBJECT (item));
+		bonobo_object_unref (BONOBO_OBJECT (item));
 	}
 
 	bonobo_container_remove (client_site->container, BONOBO_OBJECT (object));
 
+	if (client_site->bound_embeddable) {
+		bonobo_object_unref (BONOBO_OBJECT (client_site->bound_embeddable));
+		client_site->bound_embeddable = NULL;
+	}
+
 	object_class->destroy (object);
 }
 
@@ -177,10 +182,11 @@ bonobo_client_site_class_init (BonoboClientSiteClass *klass)
 static void
 bonobo_client_site_init (BonoboClientSite *client_site)
 {
+	client_site->bound_embeddable = NULL;
 }
 
-static CORBA_Object
-create_client_site (BonoboObject *object)
+CORBA_Object
+bonobo_client_site_corba_object_create (BonoboObject *object)
 {
 	POA_Bonobo_ClientSite *servant;
 	CORBA_Environment ev;
@@ -215,8 +221,8 @@ create_client_site (BonoboObject *object)
  */
 BonoboClientSite *
 bonobo_client_site_construct (BonoboClientSite  *client_site,
-			     Bonobo_ClientSite  corba_client_site,
-			     BonoboContainer   *container)
+			      Bonobo_ClientSite  corba_client_site,
+			      BonoboContainer   *container)
 {
 	g_return_val_if_fail (client_site != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_CLIENT_SITE (client_site), NULL);
@@ -256,9 +262,9 @@ bonobo_client_site_new (BonoboContainer *container)
 	g_return_val_if_fail (BONOBO_IS_CONTAINER (container), NULL);
 	
 	client_site = gtk_type_new (bonobo_client_site_get_type ());
-	corba_client_site = create_client_site (BONOBO_OBJECT (client_site));
+	corba_client_site = bonobo_client_site_corba_object_create (BONOBO_OBJECT (client_site));
 	if (corba_client_site == CORBA_OBJECT_NIL){
-		gtk_object_destroy (GTK_OBJECT (client_site));
+		bonobo_object_unref (BONOBO_OBJECT (client_site));
 		return NULL;
 	}
 
@@ -310,10 +316,10 @@ bonobo_client_site_get_type (void)
  * @client_site.
  */
 gboolean
-bonobo_client_site_bind_embeddable (BonoboClientSite *client_site, BonoboObjectClient *object)
+bonobo_client_site_bind_embeddable (BonoboClientSite   *client_site,
+				    BonoboObjectClient *object)
 {
-	CORBA_Object corba_object;
-	BonoboObject *bonobo_object;
+	CORBA_Object embeddable_object;
 	CORBA_Environment ev;
 	
 	g_return_val_if_fail (client_site != NULL, FALSE);
@@ -321,35 +327,37 @@ bonobo_client_site_bind_embeddable (BonoboClientSite *client_site, BonoboObjectC
 	g_return_val_if_fail (BONOBO_IS_CLIENT_SITE (client_site), FALSE);
 	g_return_val_if_fail (BONOBO_IS_OBJECT_CLIENT (object), FALSE);
 
-	CORBA_exception_init (&ev);
-
-	bonobo_object = BONOBO_OBJECT (object);
-
-	corba_object = bonobo_object_client_query_interface (
+	embeddable_object = bonobo_object_client_query_interface (
 		object, "IDL:Bonobo/Embeddable:1.0", NULL);
 
-	if (corba_object == CORBA_OBJECT_NIL) {
-		CORBA_exception_free (&ev);
+	if (embeddable_object == CORBA_OBJECT_NIL)
 		return FALSE;
-	}
+
+	CORBA_exception_init (&ev);
+
+	/* The QI adds a ref */
+	Bonobo_Unknown_unref (bonobo_object_corba_objref (
+		BONOBO_OBJECT (object)), &ev);
 
 	Bonobo_Embeddable_set_client_site (
-		corba_object, 
+		embeddable_object, 
 		bonobo_object_corba_objref (BONOBO_OBJECT (client_site)),
 		&ev);
 		
-	if (ev._major != CORBA_NO_EXCEPTION){
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		bonobo_object_check_env (BONOBO_OBJECT (object),
+					 embeddable_object, &ev);
 		CORBA_exception_free (&ev);
-		bonobo_object_check_env (bonobo_object, corba_object, &ev);
 		return FALSE;
 	}
-
-	client_site->bound_object = object;
-
-	Bonobo_Unknown_unref (bonobo_object_corba_objref (bonobo_object), &ev);
-	
 	CORBA_exception_free (&ev);
 
+	if (client_site->bound_embeddable)
+		bonobo_object_unref (BONOBO_OBJECT (client_site->bound_embeddable));
+
+	client_site->bound_embeddable = bonobo_object_client_from_corba (embeddable_object);
+	bonobo_object_client_ref (client_site->bound_embeddable, NULL);
+
 	return TRUE;
 }
 
@@ -364,14 +372,32 @@ bonobo_client_site_bind_embeddable (BonoboClientSite *client_site, BonoboObjectC
 BonoboObjectClient *
 bonobo_client_site_get_embeddable (BonoboClientSite *client_site)
 {
-	g_return_val_if_fail (client_site != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_CLIENT_SITE (client_site), NULL);
+	g_return_val_if_fail (
+		BONOBO_IS_CLIENT_SITE (client_site), NULL);
 
-	return client_site->bound_object;
+	return client_site->bound_embeddable;
+}
+
+/**
+ * bonobo_client_site_get_container:
+ * @client_site: A BonoboClientSite object which is bound to a remote
+ * BonoboObject server.
+ *
+ * Returns: The BonoboObjectClient object which corresponds to the
+ * remote BonoboObject to which @client_site is bound.
+ **/
+BonoboContainer *
+bonobo_client_site_get_container (BonoboClientSite *client_site)
+{
+	g_return_val_if_fail (
+		BONOBO_IS_CLIENT_SITE (client_site), NULL);
+
+	return client_site->container;
 }
 
 static void
-bonobo_client_site_view_frame_destroy (BonoboViewFrame *view_frame, BonoboClientSite *client_site)
+bonobo_client_site_view_frame_destroy (BonoboViewFrame  *view_frame,
+				       BonoboClientSite *client_site)
 {
 	/*
 	 * Remove this view frame.
@@ -401,7 +427,7 @@ bonobo_client_site_new_view_full (BonoboClientSite *client_site,
 				  gboolean          visible_cover,
 				  gboolean          active_view)
 {
-	BonoboObjectClient *server_object;
+	Bonobo_Embeddable server_object;
 	BonoboViewFrame *view_frame;
 	BonoboWrapper *wrapper;
 	Bonobo_View view;
@@ -410,9 +436,7 @@ bonobo_client_site_new_view_full (BonoboClientSite *client_site,
 
 	g_return_val_if_fail (client_site != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_CLIENT_SITE (client_site), NULL);
-	g_return_val_if_fail (client_site->bound_object != NULL, NULL);
-
-	server_object = client_site->bound_object;
+	g_return_val_if_fail (client_site->bound_embeddable != NULL, NULL);
 
 	/*
 	 * 1. Create the view frame.
@@ -425,15 +449,17 @@ bonobo_client_site_new_view_full (BonoboClientSite *client_site,
 	/*
 	 * 2. Now, create the view.
 	 */
+	server_object = bonobo_object_corba_objref (BONOBO_OBJECT (client_site->bound_embeddable));
 	CORBA_exception_init (&ev);
  	view = Bonobo_Embeddable_new_view (
-		bonobo_object_corba_objref (BONOBO_OBJECT (server_object)),
+		server_object,
 		bonobo_object_corba_objref (BONOBO_OBJECT (view_frame)),
 		&ev);
+
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		bonobo_object_check_env (
 			BONOBO_OBJECT (client_site),
-			bonobo_object_corba_objref (BONOBO_OBJECT (server_object)),
+			server_object,
 			&ev);
 		bonobo_object_unref (BONOBO_OBJECT (view_frame));
 		CORBA_exception_free (&ev);
@@ -441,6 +467,7 @@ bonobo_client_site_new_view_full (BonoboClientSite *client_site,
 	}
 
 	bonobo_view_frame_bind_to_view (view_frame, view);
+	Bonobo_Unknown_unref (view, &ev);
 	CORBA_Object_release (view, &ev);
 	
 	/*
@@ -450,7 +477,8 @@ bonobo_client_site_new_view_full (BonoboClientSite *client_site,
 	client_site->view_frames = g_list_prepend (client_site->view_frames, view_frame);
 	
 	gtk_signal_connect (GTK_OBJECT (view_frame), "destroy",
-			    GTK_SIGNAL_FUNC (bonobo_client_site_view_frame_destroy), client_site);
+			    GTK_SIGNAL_FUNC (bonobo_client_site_view_frame_destroy),
+			    client_site);
 
 	CORBA_exception_free (&ev);		
 	return view_frame;
@@ -495,11 +523,11 @@ bonobo_client_site_new_item (BonoboClientSite *client_site, GnomeCanvasGroup *gr
 		
 	g_return_val_if_fail (client_site != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_CLIENT_SITE (client_site), NULL);
-	g_return_val_if_fail (client_site->bound_object != NULL, NULL);
+	g_return_val_if_fail (client_site->bound_embeddable != NULL, NULL);
 	g_return_val_if_fail (group != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_CANVAS_GROUP (group), NULL);
 
-	server_object = client_site->bound_object;
+	server_object = client_site->bound_embeddable;
 
 	item = bonobo_canvas_item_new (group, server_object);
 
@@ -529,28 +557,26 @@ GList *
 bonobo_client_site_get_verbs (BonoboClientSite *client_site)
 {
 	Bonobo_Embeddable_verb_list *list;
-	Bonobo_Embeddable object;
-	BonoboObjectClient *server_object;
-	BonoboObject *gobject;
 	CORBA_Environment ev;
+	CORBA_Object server_object;
 	GList *l;
 	int i;
 	
 	g_return_val_if_fail (client_site != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_CLIENT_SITE (client_site), NULL);
-	g_return_val_if_fail (client_site->bound_object != NULL, NULL);
+	g_return_val_if_fail (client_site->bound_embeddable != NULL, NULL);
 
 	CORBA_exception_init (&ev);
 
-	server_object = client_site->bound_object;
+	server_object = bonobo_object_corba_objref (
+		BONOBO_OBJECT (client_site->bound_embeddable));
+	if (server_object == CORBA_OBJECT_NIL)
+		return NULL;
 
-	gobject = BONOBO_OBJECT (server_object);
-	object = (Bonobo_Embeddable) bonobo_object_corba_objref (gobject);
-
-	list = Bonobo_Embeddable_get_verb_list (object, &ev);
+	list = Bonobo_Embeddable_get_verb_list (server_object, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION){
-		bonobo_object_check_env (BONOBO_OBJECT (client_site), object, &ev);
+		bonobo_object_check_env (BONOBO_OBJECT (client_site), server_object, &ev);
 		CORBA_exception_free (&ev);
 		return NULL;
 	}
