@@ -34,10 +34,10 @@ struct _BonoboClientSite {
 	BonoboObject base;
 
 	BonoboContainer    *container;
-	BonoboObjectClient *bound_object;
-	GList		  *view_frames;
-	GList             *canvas_items;
-	int               child_shown:1;
+	BonoboObjectClient *bound_embeddable; /* IDL:Bonobo/Embeddable:1.0 */
+	GList		   *view_frames;
+	GList              *canvas_items;
+	unsigned int        child_shown:1;
 
 	BonoboClientSitePrivate *priv;
 };
@@ -51,6 +51,7 @@ typedef struct {
 } BonoboClientSiteClass;
 
 GtkType                     bonobo_client_site_get_type         (void);
+Bonobo_ClientSite           bonobo_client_site_corba_object_create (BonoboObject *object);
 BonoboClientSite           *bonobo_client_site_new              (BonoboContainer    *container);
 BonoboClientSite           *bonobo_client_site_construct        (BonoboClientSite   *client_site,
 								 Bonobo_ClientSite   corba_client_site,
@@ -58,6 +59,7 @@ BonoboClientSite           *bonobo_client_site_construct        (BonoboClientSit
 gboolean                    bonobo_client_site_bind_embeddable  (BonoboClientSite   *client_site,
 								 BonoboObjectClient *object);
 BonoboObjectClient         *bonobo_client_site_get_embeddable   (BonoboClientSite   *client_site);
+BonoboContainer            *bonobo_client_site_get_container    (BonoboClientSite   *client_site);
 
 /*
  * Proxy/Utility functions.
@@ -74,8 +76,6 @@ GList                      *bonobo_client_site_get_verbs        (BonoboClientSit
 void                        bonobo_client_site_free_verbs       (GList              *verb_list);
 POA_Bonobo_ClientSite__epv *bonobo_client_site_get_epv          (void);
 
-extern POA_Bonobo_ClientSite__vepv bonobo_client_site_vepv;
-
 END_GNOME_DECLS
 
 #endif
