@@ -19,7 +19,7 @@
 #include <gdk/gdkprivate.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdktypes.h>
-#include <gtk/gtksocket.h>
+#include <bonobo/bonobo-socket.h>
 
 enum {
 	ACTIVATED,
@@ -253,7 +253,7 @@ bonobo_control_frame_construct (BonoboControlFrame  *control_frame,
 	 * Now create the GtkSocket which will be used to embed
 	 * the Control.
 	 */
-	control_frame->priv->socket = gtk_socket_new ();
+	control_frame->priv->socket = bonobo_socket_new ();
 	gtk_widget_show (control_frame->priv->socket);
 
 	/*
@@ -283,7 +283,7 @@ bonobo_control_frame_construct (BonoboControlFrame  *control_frame,
 	 * container is needed solely for the sake of bypassing
 	 * plug/socket in the local case.
 	 */
-	control_frame->priv->container = gtk_hbox_new (1, 0);
+	control_frame->priv->container = gtk_hbox_new (FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (control_frame->priv->container), 0);
 	gtk_box_pack_end (GTK_BOX (control_frame->priv->container),
 			  control_frame->priv->socket,
@@ -321,7 +321,7 @@ bonobo_control_frame_new (Bonobo_UIHandler uih)
 
 	corba_control_frame = create_bonobo_control_frame (BONOBO_OBJECT (control_frame));
 	if (corba_control_frame == CORBA_OBJECT_NIL) {
-		gtk_object_destroy (GTK_OBJECT (control_frame));
+		bonobo_object_unref (BONOBO_OBJECT (control_frame));
 		return NULL;
 	}
 
@@ -333,7 +333,7 @@ bonobo_control_frame_destroy (GtkObject *object)
 {
 	BonoboControlFrame *control_frame = BONOBO_CONTROL_FRAME (object);
 
-	if (control_frame->priv->control != CORBA_OBJECT_NIL){
+	if (control_frame->priv->control != CORBA_OBJECT_NIL) {
 		CORBA_Environment ev;
 
 		CORBA_exception_init (&ev);
@@ -704,6 +704,7 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *control_frame, Bonobo_
 {
 	CORBA_Environment ev;
 
+	g_return_if_fail (control != CORBA_OBJECT_NIL);
 	g_return_if_fail (BONOBO_IS_CONTROL_FRAME (control_frame));
 
 	/*
@@ -711,6 +712,10 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *control_frame, Bonobo_
 	 */
 	CORBA_exception_init (&ev);
 	Bonobo_Control_ref (control, &ev);
+
+	if (control_frame->priv->control != CORBA_OBJECT_NIL)
+		g_warning ("FIXME: leaking control reference");
+
 	control_frame->priv->control = CORBA_Object_duplicate (control, &ev);
 	CORBA_exception_free (&ev);
 
@@ -719,8 +724,8 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *control_frame, Bonobo_
 	 */
 	CORBA_exception_init (&ev);
 	Bonobo_Control_set_frame (control,
-				 bonobo_object_corba_objref (BONOBO_OBJECT (control_frame)),
-				 &ev);
+				  bonobo_object_corba_objref (BONOBO_OBJECT (control_frame)),
+				  &ev);
 	if (ev._major != CORBA_NO_EXCEPTION)
 		bonobo_object_check_env (BONOBO_OBJECT (control_frame), control, &ev);
 	CORBA_exception_free (&ev);
@@ -729,10 +734,9 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *control_frame, Bonobo_
 	 * If the socket is realized, then we transfer the
 	 * window ID to the remote control.
 	 */
-	if (GTK_WIDGET_REALIZED (control_frame->priv->socket)) {
+	if (GTK_WIDGET_REALIZED (control_frame->priv->socket))
 		bonobo_control_frame_set_remote_window (control_frame->priv->socket,
-						       control_frame);
-	}
+							control_frame);
 }
 
 /**
