@@ -12,8 +12,8 @@
 #include <stdlib.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmarshal.h>
-#include <gtk/gtkplug.h>
 #include <bonobo/bonobo-main.h>
+#include <bonobo/bonobo-plug.h>
 #include <bonobo/bonobo-control.h>
 #include <gdk/gdkprivate.h>
 
@@ -131,7 +131,7 @@ bonobo_control_plug_destroy_event_cb (GtkWidget   *plug,
 	/*
 	 * Destroy this plug's BonoboControl.
 	 */
-	bonobo_object_destroy (BONOBO_OBJECT (control));
+	bonobo_object_unref (BONOBO_OBJECT (control));
 }
 
 /*
@@ -156,7 +156,7 @@ bonobo_control_plug_destroy_cb (GtkWidget *plug,
 	 * destroy it later.  It will get destroyed on its
 	 * own.
 	 */
-	control->priv->plug            = NULL;
+	control->priv->plug = NULL;
 }
 
 
@@ -280,7 +280,8 @@ impl_Bonobo_Control_set_window (PortableServer_Servant   servant,
 		old_plug            = control->priv->plug;
 
 		/* Create the new plug */
-		control->priv->plug = gtk_plug_new (x11_id);
+		control->priv->plug = bonobo_plug_new (x11_id);
+
 		gtk_signal_connect (GTK_OBJECT (control->priv->plug), "destroy_event",
 				    GTK_SIGNAL_FUNC (bonobo_control_plug_destroy_event_cb), control);
 		gtk_signal_connect (GTK_OBJECT (control->priv->plug), "destroy",
@@ -303,6 +304,7 @@ impl_Bonobo_Control_set_window (PortableServer_Servant   servant,
 		}
 
 		gtk_widget_show (control->priv->plug);
+		
 	} else {
 		GtkWidget *socket_parent;
 
@@ -485,8 +487,8 @@ bonobo_control_new (GtkWidget *widget)
 	control = gtk_type_new (bonobo_control_get_type ());
 
 	corba_control = bonobo_control_corba_object_create (BONOBO_OBJECT (control));
-	if (corba_control == CORBA_OBJECT_NIL){
-		gtk_object_destroy (GTK_OBJECT (control));
+	if (corba_control == CORBA_OBJECT_NIL) {
+		bonobo_object_unref (BONOBO_OBJECT (control));
 		return NULL;
 	}
 	
@@ -721,20 +723,31 @@ static void
 bonobo_control_destroy (GtkObject *object)
 {
 	BonoboControl *control = BONOBO_CONTROL (object);
+	CORBA_Environment ev;
+
+	CORBA_exception_init (&ev);
+
+	if (control->priv->active)
+		Bonobo_ControlFrame_activated (control->priv->control_frame,
+					       FALSE, &ev);
+
+	CORBA_Object_release (control->priv->control_frame, &ev);
+
+	CORBA_exception_free (&ev);
 	
 	/*
 	 * If we have a UIHandler, destroy it.
 	 */
 	if (control->priv->uih != NULL) {
 		bonobo_ui_handler_unset_container (control->priv->uih);
-		bonobo_object_destroy (BONOBO_OBJECT (control->priv->uih));
+		bonobo_object_unref (BONOBO_OBJECT (control->priv->uih));
 	}
 
 	/*
 	 * Destroy the control's top-level widget.
 	 */
 	if (control->priv->widget)
-		gtk_object_destroy (GTK_OBJECT (control->priv->widget));
+		gtk_object_unref (GTK_OBJECT (control->priv->widget));
 
 	/*
 	 * If the plug still exists, destroy it.  The plug might not
@@ -805,8 +818,9 @@ bonobo_control_set_control_frame (BonoboControl *control, Bonobo_ControlFrame co
 
 	CORBA_exception_init (&ev);
 
-	Bonobo_Unknown_ref (control_frame, &ev);
-
+	if (control->priv->control_frame != CORBA_OBJECT_NIL)
+		CORBA_Object_release (control->priv->control_frame, &ev);
+	
 	control->priv->control_frame = CORBA_Object_duplicate (control_frame, &ev);
 	
 	CORBA_exception_free (&ev);
@@ -961,7 +975,7 @@ bonobo_control_get_remote_ui_handler (BonoboControl *control)
  */
 void
 bonobo_control_activate_notify (BonoboControl *control,
-			       gboolean      activated)
+				gboolean      activated)
 {
 	CORBA_Environment ev;
 
@@ -1019,6 +1033,8 @@ static void
 bonobo_control_init (BonoboControl *control)
 {
 	control->priv = g_new0 (BonoboControlPrivate, 1);
+
+	control->priv->control_frame = CORBA_OBJECT_NIL;
 }
 
 /**
@@ -1064,6 +1080,7 @@ bonobo_control_set_property (BonoboControl       *control,
 	g_return_if_fail (BONOBO_IS_CONTROL (control));
 	g_return_if_fail (first_prop != NULL);
 
+	bonobo_object_ref (BONOBO_OBJECT (control->priv->propbag));
 	bag = (Bonobo_PropertyBag)
 		bonobo_object_corba_objref (BONOBO_OBJECT (control->priv->propbag));
 	cl = bonobo_property_bag_client_new (bag);
@@ -1071,6 +1088,8 @@ bonobo_control_set_property (BonoboControl       *control,
 	if ((err = bonobo_property_bag_client_setv (cl, first_prop, args)))
 		g_warning ("Error '%s'", err);
 
+	bonobo_object_unref (BONOBO_OBJECT (cl));
+
 	va_end (args);
 }
 
@@ -1096,5 +1115,7 @@ bonobo_control_get_property (BonoboControl       *control,
 	if ((err = bonobo_property_bag_client_getv (cl, first_prop, args)))
 		g_warning ("Error '%s'", err);
 
+	bonobo_object_unref (BONOBO_OBJECT (cl));
+
 	va_end (args);
 }
