@@ -7,6 +7,8 @@
 #include <bonobo/Bonobo.h>
 
 BEGIN_GNOME_DECLS
+
+#undef BONOBO_OBJECT_DEBUG
  
 #define BONOBO_OBJECT_TYPE        (bonobo_object_get_type ())
 #define BONOBO_OBJECT(o)          (GTK_CHECK_CAST ((o), BONOBO_OBJECT_TYPE, BonoboObject))
@@ -29,7 +31,7 @@ typedef struct _BonoboObjectPrivate BonoboObjectPrivate;
 typedef struct {
 	GtkObject base;
 
-	CORBA_Object          corba_objref;
+	Bonobo_Unknown        corba_objref;
 	gpointer              servant;
 	BonoboObjectPrivate   *priv;
 } BonoboObject;
@@ -53,24 +55,31 @@ BonoboObject            *bonobo_object_from_servant           (PortableServer_Se
 void                     bonobo_object_bind_to_servant        (BonoboObject           *object,
 							       void                   *servant);
 PortableServer_Servant   bonobo_object_get_servant            (BonoboObject           *object);
-CORBA_Object             bonobo_object_activate_servant       (BonoboObject           *object,
+Bonobo_Unknown           bonobo_object_activate_servant       (BonoboObject           *object,
 							       void                   *servant);
 void                     bonobo_object_add_interface          (BonoboObject           *object,
 							       BonoboObject           *newobj);
 BonoboObject            *bonobo_object_query_local_interface  (BonoboObject           *object,
 							       const char             *repo_id);
-CORBA_Object             bonobo_object_query_interface        (BonoboObject           *object,
+Bonobo_Unknown           bonobo_object_query_interface        (BonoboObject           *object,
 							       const char             *repo_id);
-CORBA_Object             bonobo_object_corba_objref           (BonoboObject           *object);
+Bonobo_Unknown           bonobo_object_corba_objref           (BonoboObject           *object);
 
 /*
  * Gnome Object Life Cycle
  */
 void                     bonobo_object_ref                    (BonoboObject           *object);
 void                     bonobo_object_unref                  (BonoboObject           *object);
-void                     bonobo_object_destroy                (BonoboObject           *object);
 POA_Bonobo_Unknown__epv *bonobo_object_get_epv                (void);
-
+void                     bonobo_object_init                   (void);
+void                     bonobo_object_trace_refs             (BonoboObject *object,
+							       const char   *fn,
+							       int           line,
+							       gboolean      ref);
+#ifdef BONOBO_OBJECT_DEBUG
+#	define           bonobo_object_ref(o)   G_STMT_START{bonobo_object_trace_refs((o),G_GNUC_PRETTY_FUNCTION,__LINE__,TRUE);}G_STMT_END
+#	define           bonobo_object_unref(o) G_STMT_START{bonobo_object_trace_refs((o),G_GNUC_PRETTY_FUNCTION,__LINE__,FALSE);}G_STMT_END
+#endif	/* BONOBO_OBJECT_DEBUG */
 
 /*
  * Error checking
@@ -79,19 +88,17 @@ void                     bonobo_object_check_env              (BonoboObject
 							       CORBA_Object            corba_object,
 							       CORBA_Environment      *ev);
 
-#define BONOBO_OBJECT_CHECK(o,c,e) do { \
-                                        if ((e)->_major != CORBA_NO_EXCEPTION) { \
-                                        	bonobo_object_check_env(o,c,e);     \
-			        } while (0)
+#define BONOBO_OBJECT_CHECK(o,c,e)	\
+			G_STMT_START {				\
+			if ((e)->_major != CORBA_NO_EXCEPTION)	\
+				bonobo_object_check_env(o,c,e);	\
+			} G_STMT_END
 
 /*
  * Others
  */
 gboolean  gnome_unknown_ping (Bonobo_Unknown object);
 
-/* CORBA default vector methods we provide */
-extern POA_Bonobo_Unknown__vepv          bonobo_object_vepv;
-
 END_GNOME_DECLS
 
 #endif
