@@ -35,6 +35,11 @@ GtkType	    bonobo_selector_get_type		(void);
 
 GtkWidget  *bonobo_selector_new			(const gchar *title,
 							 const gchar **interfaces_required);
+
+gchar	   *bonobo_selector_get_selected_id   (BonoboSelector *sel);
+gchar	   *gnome_bonobo_select_id                  (const gchar *title,
+							 const gchar **interfaces_required);
+
 gchar	   *bonobo_selector_get_selected_goad_id  (BonoboSelector *sel);
 gchar	   *gnome_bonobo_select_goad_id			(const gchar *title,
 							 const gchar **interfaces_required);
