@@ -27,8 +27,9 @@ mem_truncate (BonoboStream *stream,
 		return;
 
 	newp = g_realloc (smem->buffer, new_size);
-	if (newp == NULL){
-		g_warning ("Signal exception");
+	if (!newp) {
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+				     ex_Bonobo_Stream_NoPermission, NULL);
 		return;
 	}
 
@@ -70,7 +71,7 @@ mem_write (BonoboStream *stream, const Bonobo_Stream_iobuf *buffer,
 	return len;
 }
 
-static CORBA_long
+static void
 mem_read (BonoboStream *stream, CORBA_long count,
 	  Bonobo_Stream_iobuf ** buffer,
 	  CORBA_Environment *ev)
@@ -88,8 +89,6 @@ mem_read (BonoboStream *stream, CORBA_long count,
 	memcpy ((*buffer)->_buffer, smem->buffer + smem->pos, count);
 
 	smem->pos += count;
-	
-	return count;
 }
 
 static CORBA_long
@@ -147,7 +146,7 @@ mem_copy_to  (BonoboStream *stream,
 	
 	/* create the output file */
 	fd_out = creat(dest, 0666);
-	if(fd_out == -1) {
+	if (fd_out == -1) {
 		g_warning ("unable to create output file");
 		return;
 	}
@@ -186,6 +185,22 @@ mem_destroy (GtkObject *object)
 	GTK_OBJECT_CLASS (bonobo_stream_mem_parent_class)->destroy (object);
 }
 
+static char *
+mem_get_buffer (BonoboStreamMem *stream_mem)
+{
+	g_return_val_if_fail (BONOBO_IS_STREAM_MEM (stream_mem), NULL);
+
+	return stream_mem->buffer;
+}
+
+static size_t
+mem_get_size (BonoboStreamMem *stream_mem)
+{
+	g_return_val_if_fail (BONOBO_IS_STREAM_MEM (stream_mem), NULL);
+
+	return stream_mem->size;
+}
+
 static void
 bonobo_stream_mem_class_init (BonoboStreamMemClass *klass)
 {
@@ -196,12 +211,15 @@ bonobo_stream_mem_class_init (BonoboStreamMemClass *klass)
 
 	object_class->destroy = mem_destroy;
 	
-	sclass->write    = mem_write;
-	sclass->read     = mem_read;
-	sclass->seek     = mem_seek;
-	sclass->truncate = mem_truncate;
-	sclass->copy_to  = mem_copy_to;
-	sclass->commit   = mem_commit;
+	sclass->write     = mem_write;
+	sclass->read      = mem_read;
+	sclass->seek      = mem_seek;
+	sclass->truncate  = mem_truncate;
+	sclass->copy_to   = mem_copy_to;
+	sclass->commit    = mem_commit;
+
+	klass->get_buffer = mem_get_buffer;
+	klass->get_size   = mem_get_size;
 }
 
 /**
@@ -232,26 +250,31 @@ bonobo_stream_mem_get_type (void)
 	return type;
 }
 
-static Bonobo_Stream
-create_bonobo_stream_mem (BonoboObject *object)
+BonoboStreamMem *
+bonobo_stream_mem_construct (BonoboStreamMem *stream_mem,
+			     Bonobo_Stream    corba_stream,
+			     const char      *buffer,
+			     size_t           size,
+			     gboolean         read_only,
+			     gboolean         resizable)
 {
-	POA_Bonobo_Stream *servant;
-	CORBA_Environment ev;
+	g_return_val_if_fail (corba_stream != CORBA_OBJECT_NIL, NULL);
+	g_return_val_if_fail (BONOBO_IS_STREAM_MEM (stream_mem), NULL);
 
-	servant = (POA_Bonobo_Stream *) g_new0 (BonoboObjectServant, 1);
-	servant->vepv = &bonobo_stream_vepv;
+	if (buffer == NULL) {
+		stream_mem->buffer = g_malloc (size);
+		memset (stream_mem->buffer, 0, size);
+	} else
+		stream_mem->buffer = g_memdup (buffer, size);
 
-	CORBA_exception_init (&ev);
+	stream_mem->size = size;
+	stream_mem->pos = 0;
+	stream_mem->read_only = read_only;
+	stream_mem->resizable = resizable;
 
-	POA_Bonobo_Stream__init ((PortableServer_Servant) servant, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION){
-                g_free (servant);
-		CORBA_exception_free (&ev);
-                return CORBA_OBJECT_NIL;
-        }
-
-	CORBA_exception_free (&ev);
-	return (Bonobo_Stream) bonobo_object_activate_servant (object, servant);
+	return BONOBO_STREAM_MEM (
+		bonobo_object_construct (BONOBO_OBJECT (stream_mem),
+					 corba_stream));
 }
 
 /**
@@ -277,7 +300,7 @@ create_bonobo_stream_mem (BonoboObject *object)
  * Returns: the constructed BonoboStream object
  **/
 BonoboStream *
-bonobo_stream_mem_create (char *buffer, size_t size,
+bonobo_stream_mem_create (const char *buffer, size_t size,
 			  gboolean read_only, gboolean resizable)
 {
 	BonoboStreamMem *stream_mem;
@@ -287,26 +310,18 @@ bonobo_stream_mem_create (char *buffer, size_t size,
 	if (stream_mem == NULL)
 		return NULL;
 
-	if (buffer == NULL) {
-		stream_mem->buffer = g_malloc (size);
-		memset (buffer, 0, size);
-	} else
-		stream_mem->buffer = g_memdup (buffer, size);
 
-	stream_mem->size = size;
-	stream_mem->pos = 0;
-	stream_mem->read_only = read_only;
-	stream_mem->resizable = resizable;
+	corba_stream = bonobo_stream_corba_object_create (
+		BONOBO_OBJECT (stream_mem));
 
-	corba_stream = create_bonobo_stream_mem (BONOBO_OBJECT (stream_mem));
-
-	if (corba_stream == CORBA_OBJECT_NIL){
-		gtk_object_destroy (GTK_OBJECT (stream_mem));
+	if (corba_stream == CORBA_OBJECT_NIL) {
+		bonobo_object_unref (BONOBO_OBJECT (stream_mem));
 		return NULL;
 	}
 
-	bonobo_object_construct (BONOBO_OBJECT (stream_mem), corba_stream);
-	return BONOBO_STREAM (stream_mem);
+	return BONOBO_STREAM (bonobo_stream_mem_construct (
+		stream_mem, corba_stream, buffer, size,
+		read_only, resizable));
 }
 
 /**
@@ -323,5 +338,22 @@ bonobo_stream_mem_create (char *buffer, size_t size,
 const char *
 bonobo_stream_mem_get_buffer (BonoboStreamMem *stream_mem)
 {
-	return stream_mem->buffer;
+	return BONOBO_STREAM_MEM_CLASS(
+		GTK_OBJECT(stream_mem)->klass)->get_buffer (stream_mem);
+}
+
+/**
+ * bonobo_stream_mem_get_size:
+ * @stream_mem: a BonoboStreamMem
+ *
+ * Returns the size of the data associated with a BonoboStreamMem
+ * see bonobo_stream_mem_get_buffer
+ *
+ * Return value: the size.
+ **/
+size_t
+bonobo_stream_mem_get_size (BonoboStreamMem *stream_mem)
+{
+	return BONOBO_STREAM_MEM_CLASS(
+		GTK_OBJECT(stream_mem)->klass)->get_size (stream_mem);
 }
