@@ -34,7 +34,7 @@ typedef struct _BonoboUIHandlerToolbarItem BonoboUIHandlerToolbarItem;
  * callback routines, whose arguments are (GtkWidget *menu_item, void
  * *user_data).
  */
-typedef void (*BonoboUIHandlerCallbackFunc) (BonoboUIHandler *uih, void *user_data, const char *path);
+typedef void (*BonoboUIHandlerCallback) (BonoboUIHandler *uih, void *user_data, const char *path);
 
 /*
  * The BonoboUIHandler object.
@@ -106,9 +106,19 @@ typedef enum {
 	BONOBO_UI_HANDLER_MENU_RADIOITEM,
 	BONOBO_UI_HANDLER_MENU_RADIOGROUP,
 	BONOBO_UI_HANDLER_MENU_TOGGLEITEM,
-	BONOBO_UI_HANDLER_MENU_SEPARATOR
+	BONOBO_UI_HANDLER_MENU_SEPARATOR,
+	BONOBO_UI_HANDLER_MENU_PLACEHOLDER
 } BonoboUIHandlerMenuItemType;
 
+/*
+ *  A hack to allow placeholders to be put into a gnome-app-helper
+ * structure, so that they can be used trivialy.
+ */
+#define BONOBO_APP_UI_PLACEHOLDER (GNOME_APP_UI_SUBTREE_STOCK + 0x100)
+#define BONOBOUIINFO_PLACEHOLDER(label) \
+	{ BONOBO_APP_UI_PLACEHOLDER, label, NULL, NULL, NULL, NULL, \
+		GNOME_APP_PIXMAP_NONE, NULL, 0, (GdkModifierType) 0, NULL }
+
 struct _BonoboUIHandlerMenuItem {
 	char			        *path;
 
@@ -135,8 +145,9 @@ struct _BonoboUIHandlerMenuItem {
 	/*
 	 * The callback function and closure.
 	 */
-	BonoboUIHandlerCallbackFunc	 callback;
+	BonoboUIHandlerCallback	         callback;
 	gpointer			 callback_data;
+	GDestroyNotify                   callback_data_destroy_notify;
 };
 
 typedef enum {
@@ -172,8 +183,9 @@ struct _BonoboUIHandlerToolbarItem {
 	/*
 	 * The callback function and closure.
 	 */
-	BonoboUIHandlerCallbackFunc	 callback;
+	BonoboUIHandlerCallback	         callback;
 	gpointer			 callback_data;
+	GDestroyNotify                   callback_data_destroy_notify;
 };
 
 /*
@@ -229,14 +241,14 @@ void			 bonobo_ui_handler_menu_new			(BonoboUIHandler *uih, const char *path,
 									 int pos, BonoboUIHandlerPixmapType pixmap_type,
 									 gpointer pixmap_data,
 									 guint accelerator_key, GdkModifierType ac_mods,
-									 BonoboUIHandlerCallbackFunc callback,
+									 BonoboUIHandlerCallback callback,
 									 gpointer callback_data);
 void			 bonobo_ui_handler_menu_new_item		(BonoboUIHandler *uih, const char *path,
 									 const char *label, const char *hint,
 									 int pos, BonoboUIHandlerPixmapType pixmap_type,
 									 gpointer pixmap_data,
 									 guint accelerator_key, GdkModifierType ac_mods,
-									 BonoboUIHandlerCallbackFunc callback,
+									 BonoboUIHandlerCallback callback,
 									 gpointer callback_data);
 void			 bonobo_ui_handler_menu_new_subtree		(BonoboUIHandler *uih, const char *path,
 									 const char *label, const char *hint, int pos,
@@ -244,16 +256,17 @@ void			 bonobo_ui_handler_menu_new_subtree		(BonoboUIHandler *uih, const char *p
 									 gpointer pixmap_data, guint accelerator_key,
 									 GdkModifierType ac_mods);
 void			 bonobo_ui_handler_menu_new_separator		(BonoboUIHandler *uih, const char *path, int pos);
+void			 bonobo_ui_handler_menu_new_placeholder		(BonoboUIHandler *uih, const char *path);
 void			 bonobo_ui_handler_menu_new_radiogroup		(BonoboUIHandler *uih, const char *path);
 void			 bonobo_ui_handler_menu_new_radioitem		(BonoboUIHandler *uih, const char *path,
 									 const char *label, const char *hint, int pos,
 									 guint accelerator_key, GdkModifierType ac_mods,
-									 BonoboUIHandlerCallbackFunc callback,
+									 BonoboUIHandlerCallback callback,
 									 gpointer callback_data);
 void			 bonobo_ui_handler_menu_new_toggleitem		(BonoboUIHandler *uih, const char *path,
 									 const char *label, const char *hint, int pos,
 									 guint accelerator_key, GdkModifierType ac_mods,
-									 BonoboUIHandlerCallbackFunc callback,
+									 BonoboUIHandlerCallback callback,
 									 gpointer callback_data);
 void			 bonobo_ui_handler_menu_add_one			(BonoboUIHandler *uih, const char *parent_path,
 									 BonoboUIHandlerMenuItem *item);
@@ -266,13 +279,14 @@ BonoboUIHandlerMenuItem	*bonobo_ui_handler_menu_fetch_one		(BonoboUIHandler *uih
 BonoboUIHandlerMenuItem	*bonobo_ui_handler_menu_fetch_list		(BonoboUIHandler *uih, const char *path);
 BonoboUIHandlerMenuItem	*bonobo_ui_handler_menu_fetch_tree		(BonoboUIHandler *uih, const char *path);
 
+gboolean                 bonobo_ui_handler_menu_path_exists             (BonoboUIHandler *uih, const char *path);
 void			 bonobo_ui_handler_menu_free_one			(BonoboUIHandlerMenuItem *item);
 void			 bonobo_ui_handler_menu_free_list		(BonoboUIHandlerMenuItem *item);
 void			 bonobo_ui_handler_menu_free_tree		(BonoboUIHandlerMenuItem *item);
 
 
 GList			*bonobo_ui_handler_menu_fetch_by_callback	(BonoboUIHandler *uih,
-									 BonoboUIHandlerCallbackFunc callback);
+									 BonoboUIHandlerCallback callback);
 GList			*bonobo_ui_handler_menu_fetch_by_callback_data	(BonoboUIHandler *uih, gpointer callback_data);
 GList			*bonobo_ui_handler_menu_get_child_paths		(BonoboUIHandler *uih, const char *parent_path);
 
@@ -309,11 +323,14 @@ void			 bonobo_ui_handler_menu_get_accel		(BonoboUIHandler *uih, const char *pat
 									 guint *accelerator_key, GdkModifierType *ac_mods);
 
 void			 bonobo_ui_handler_menu_set_callback		(BonoboUIHandler *uih, const char *path,
-									 BonoboUIHandlerCallbackFunc callback,
-									 gpointer callback_data);
+									 BonoboUIHandlerCallback callback,
+									 gpointer callback_data,
+									 GDestroyNotify callback_data_destroy_notify);
 void			 bonobo_ui_handler_menu_get_callback		(BonoboUIHandler *uih, const char *path,
-									 BonoboUIHandlerCallbackFunc *callback,
-									 gpointer *callback_data);
+									 BonoboUIHandlerCallback *callback,
+									 gpointer *callback_data,
+									 GDestroyNotify *callback_data_destroy_notify);
+void			 bonobo_ui_handler_menu_remove_callback_no_notify (BonoboUIHandler *uih, const char *path);
 
 void			 bonobo_ui_handler_menu_set_toggle_state		(BonoboUIHandler *uih, const char *path,
 									 gboolean state);
@@ -350,7 +367,7 @@ void				 bonobo_ui_handler_toolbar_new		(BonoboUIHandler *uih, const char *path,
 									 BonoboUIHandlerPixmapType pixmap_type,
 									 gpointer pixmap_data, guint accelerator_key,
 									 GdkModifierType ac_mods,
-									 BonoboUIHandlerCallbackFunc callback,
+									 BonoboUIHandlerCallback callback,
 									 gpointer callback_data);
 void				 bonobo_ui_handler_toolbar_new_control	(BonoboUIHandler *uih, const char *path,
 									 int pos, Bonobo_Control control);
@@ -359,7 +376,7 @@ void				 bonobo_ui_handler_toolbar_new_item	(BonoboUIHandler *uih, const char *p
 									 BonoboUIHandlerPixmapType pixmap_type,
 									 gpointer pixmap_data,
 									 guint accelerator_key, GdkModifierType ac_mods,
-									 BonoboUIHandlerCallbackFunc callback,
+									 BonoboUIHandlerCallback callback,
 									 gpointer callback_data);
 void				 bonobo_ui_handler_toolbar_new_separator	(BonoboUIHandler *uih, const char *path, int pos);
 void				 bonobo_ui_handler_toolbar_new_radiogroup(BonoboUIHandler *uih, const char *path);
@@ -368,14 +385,14 @@ void				 bonobo_ui_handler_toolbar_new_radioitem	(BonoboUIHandler *uih, const ch
 									 BonoboUIHandlerPixmapType pixmap_type,
 									 gpointer pixmap_data,
 									 guint accelerator_key, GdkModifierType ac_mods,
-									 BonoboUIHandlerCallbackFunc callback,
+									 BonoboUIHandlerCallback callback,
 									 gpointer callback_data);
 void				 bonobo_ui_handler_toolbar_new_toggleitem(BonoboUIHandler *uih, const char *path,
 									 const char *label, const char *hint, int pos,
 									 BonoboUIHandlerPixmapType pixmap_type,
 									 gpointer pixmap_data,
 									 guint accelerator_key, GdkModifierType ac_mods,
-									 BonoboUIHandlerCallbackFunc callback,
+									 BonoboUIHandlerCallback callback,
 									 gpointer callback_data);
 void				 bonobo_ui_handler_toolbar_add_one	(BonoboUIHandler *uih, const char *parent_path,
 									 BonoboUIHandlerToolbarItem *item);
@@ -388,7 +405,7 @@ BonoboUIHandlerToolbarItem	*bonobo_ui_handler_toolbar_fetch_one	(BonoboUIHandler
 BonoboUIHandlerToolbarItem	*bonobo_ui_handler_toolbar_fetch_list	(BonoboUIHandler *uih, const char *path);
 BonoboUIHandlerToolbarItem	*bonobo_ui_handler_toolbar_fetch_tree	(BonoboUIHandler *uih, const char *path);
 GList				*bonobo_ui_handler_toolbar_fetch_by_callback (BonoboUIHandler *uih,
-									     BonoboUIHandlerCallbackFunc callback);
+									     BonoboUIHandlerCallback callback);
 GList				*bonobo_ui_handler_toolbar_fetch_by_callback_data (BonoboUIHandler *uih,
 										  gpointer callback_data);
 GList				*bonobo_ui_handler_toolbar_get_child_paths (BonoboUIHandler *uih, const char *path);
@@ -432,12 +449,15 @@ void				 bonobo_ui_handler_toolbar_item_get_accel	(BonoboUIHandler *uih, const c
 										 guint *aaccelerator_key, GdkModifierType *ac_mods);
 
 void				 bonobo_ui_handler_toolbar_item_set_callback	(BonoboUIHandler *uih, const char *path,
-										 BonoboUIHandlerCallbackFunc callback,
-										 gpointer callback_data);
+										 BonoboUIHandlerCallback callback,
+										 gpointer callback_data,
+										 GDestroyNotify callback_data_destroy_notify);
 
 void				 bonobo_ui_handler_toolbar_item_get_callback	(BonoboUIHandler *uih, const char *path,
-										 BonoboUIHandlerCallbackFunc *callback,
-										 gpointer *callback_data);
+										 BonoboUIHandlerCallback *callback,
+										 gpointer *callback_data,
+										 GDestroyNotify *callback_data_destroy_notify);
+void			         bonobo_ui_handler_toolbar_item_remove_callback_no_notify (BonoboUIHandler *uih, const char *path);
 
 void				 bonobo_ui_handler_toolbar_item_toggle_set_state (BonoboUIHandler *uih, const char *path, gboolean state);
 gboolean			 bonobo_ui_handler_toolbar_item_toggle_get_state (BonoboUIHandler *uih, const char *path);
@@ -466,7 +486,6 @@ gboolean  bonobo_ui_handler_dock_set_sensitive  (BonoboUIHandler       *uih,
 gboolean  bonobo_ui_handler_dock_get_sensitive  (BonoboUIHandler       *uih,
 						 const char            *name);
 
-extern POA_Bonobo_UIHandler__vepv bonobo_ui_handler_vepv;
 POA_Bonobo_UIHandler__epv *bonobo_ui_handler_get_epv (void);
 
 #endif /* _BONOBO_UI_HANDLER_H_ */
