@@ -132,12 +132,11 @@ bonobo_ui_handler_dock_remove_data (BonoboUIHandler        *uih,
 		return FALSE;
 	}
 
-/*
+	g_hash_table_remove (uih->top->name_to_dock, key);
+	g_free (internal->name);
 	g_free (internal); 
-	g_free (internal->name); FIXME */
-
 	g_free (key);
-
+	
 	CORBA_exception_free (&ev);
 
 	return TRUE;
