@@ -1,3 +1,5 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+
 #ifndef __BONOBO_UIH_PRIVATE_H__
 #define __BONOBO_UIH_PRIVATE_H__
 
@@ -137,8 +139,9 @@ typedef struct {
 	 */
 	GList				*children;
 
-	BonoboUIHandlerCallbackFunc	 callback;
+	BonoboUIHandlerCallback	         callback;
 	gpointer			 callback_data;
+	GDestroyNotify                   callback_data_destroy_notify;
 } MenuItemLocalInternal;
 
 /*
@@ -189,17 +192,17 @@ struct _ToolbarToolbarInternal {
 	 * This toolbar's name. e.g. "Common"
 	 * The path for a toolbar item will be something like "/Common/Help"
 	 */
-	char			 *name;
+	char		        *name;
 
 	/*
 	 * A list of paths for the items in this toolbar.
 	 */
-	GList			 *children;
+	GList			*children;
 
 	/*
 	 * The owner for this toolbar.
 	 */
-	Bonobo_UIHandler		 uih_corba;
+	Bonobo_UIHandler         uih_corba;
 
 	GtkOrientation		 orientation;
 	GtkToolbarStyle		 style;
@@ -209,8 +212,9 @@ struct _ToolbarToolbarInternal {
 };
 
 struct _ToolbarItemLocalInternal {
-	BonoboUIHandlerCallbackFunc callback;
+	BonoboUIHandlerCallback    callback;
 	gpointer	           callback_data;
+	GDestroyNotify             callback_data_destroy_notify;
 };
 
 struct _ToolbarToolbarLocalInternal {
@@ -233,7 +237,7 @@ typedef struct {
 } ToolbarRemoteAttributeData;
 
 
-gchar    ** bonobo_ui_handler_path_tokenize (const char *path);
+gchar                      **bonobo_ui_handler_path_tokenize                          (const char *path);
 void                         bonobo_ui_handler_remote_attribute_data_free             (UIRemoteAttributeData         *attrs);
 gboolean                     bonobo_ui_handler_toplevel_check_toplevel                (BonoboUIHandler               *uih);
 void                         bonobo_ui_handler_toplevel_add_containee                 (BonoboUIHandler               *uih,
