@@ -193,7 +193,7 @@ toolbar_make_item (const char                     *path,
 		   gpointer                        pixmap_data,
 		   guint                           accelerator_key,
 		   GdkModifierType                 ac_mods,
-		   BonoboUIHandlerCallbackFunc     callback,
+		   BonoboUIHandlerCallback         callback,
 		   gpointer                        callback_data)
 {
 	BonoboUIHandlerToolbarItem *item;
@@ -237,8 +237,9 @@ toolbar_copy_item (BonoboUIHandlerToolbarItem *item)
 	copy->accelerator_key = item->accelerator_key;
 	copy->ac_mods         = item->ac_mods;
 
-	copy->callback        = item->callback;
-	copy->callback_data   = item->callback_data;
+	/* Don't copy the callback information.
+	 * Only the original holds the callback information.
+	 */
 
 	CORBA_exception_init (&ev);
 	copy->control         = CORBA_Object_duplicate (item->control, &ev);
@@ -260,6 +261,10 @@ toolbar_free_data (BonoboUIHandlerToolbarItem *item)
 	item->hint = NULL;
 
 	bonobo_ui_handler_pixmap_free_data (item->pixmap_type, item->pixmap_data);
+
+	if (item->callback_data_destroy_notify != NULL) {
+		(* item->callback_data_destroy_notify) (item->callback_data);
+	}
 }
 
 static void
@@ -465,6 +470,10 @@ static void
 toolbar_toplevel_toolbar_create_widget (BonoboUIHandler *uih, ToolbarToolbarInternal *internal)
 {
 	GtkWidget *toolbar_widget;
+	GList *l;
+	GnomeDockLayout *layout = uih->top->app->layout;
+	GnomeDockItemBehavior beh = GNOME_DOCK_ITEM_BEH_EXCLUSIVE;
+	guint band = 0;
 
 	/*
 	 * Create the toolbar widget.
@@ -479,7 +488,22 @@ toolbar_toplevel_toolbar_create_widget (BonoboUIHandler *uih, ToolbarToolbarInte
 	/*
 	 * Stuff it into the application.
 	 */
-	gnome_app_set_toolbar (uih->top->app, GTK_TOOLBAR (toolbar_widget));
+
+	if (layout) {
+		for (l = layout->items; l; l = l->next)
+			if (((GnomeDockLayoutItem *)l->data)->placement == GNOME_DOCK_TOP)
+				band++;
+	} else {
+		GnomeDock *dock = gnome_app_get_dock (uih->top->app);
+		band = g_list_length (dock->top_bands);
+	}
+	
+	if (!gnome_preferences_get_toolbar_detachable())
+		beh |= GNOME_DOCK_ITEM_BEH_LOCKED;
+
+	gnome_app_add_toolbar (uih->top->app, GTK_TOOLBAR (toolbar_widget),
+			       internal->name, beh, GNOME_DOCK_TOP, 
+			       band, 0, 0);
 
 	gtk_widget_show (toolbar_widget);
 }
@@ -553,6 +577,7 @@ toolbar_toplevel_toolbar_remove_widgets_recursive (BonoboUIHandler *uih, const c
 	ToolbarToolbarInternal *internal;
 	GtkWidget *toolbar_widget;
 	char *orig_key;
+	gpointer dummy;
 	GList *curr;
 
 	internal = toolbar_toplevel_get_toolbar (uih, name);
@@ -572,6 +597,12 @@ toolbar_toplevel_toolbar_remove_widgets_recursive (BonoboUIHandler *uih, const c
 	g_hash_table_remove (uih->top->name_to_toolbar_widget, name);
 	g_free (orig_key);
 
+	/* FIXME: Leak ToolbarToolbarInternal's like a sow */
+	g_hash_table_lookup_extended (uih->top->name_to_toolbar, name,
+				      (gpointer *) &orig_key, &dummy);
+	g_hash_table_remove (uih->top->name_to_toolbar, name);
+	g_free (orig_key);
+
 	if (GNOME_IS_DOCK_ITEM (toolbar_widget->parent)) {
 		/*
 		 * Since GnomeDock's get left floating, and since there is
@@ -667,6 +698,11 @@ toolbar_toplevel_toolbar_check_override (BonoboUIHandler *uih, const char *name)
 	if (internal == NULL)
 		return;
 
+	/*
+	 * Notification.
+	 */
+	toolbar_toplevel_toolbar_override_notify_recursive (uih, name);
+
 	/*
 	 * There is a toolbar by this name, and so it must
 	 * be overridden.
@@ -676,11 +712,6 @@ toolbar_toplevel_toolbar_check_override (BonoboUIHandler *uih, const char *name)
 	 * overridden.
 	 */
 	toolbar_toplevel_toolbar_remove_widgets_recursive (uih, name);
-
-	/*
-	 * Notification.
-	 */
-	toolbar_toplevel_toolbar_override_notify_recursive (uih, name);
 }
 
 static ToolbarToolbarInternal *
@@ -807,7 +838,6 @@ impl_Bonobo_UIHandler_toolbar_create (PortableServer_Servant  servant,
 void
 bonobo_ui_handler_create_toolbar (BonoboUIHandler *uih, const char *name)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (name != NULL);
 	g_return_if_fail (name[0] != '/');
@@ -819,13 +849,22 @@ bonobo_ui_handler_create_toolbar (BonoboUIHandler *uih, const char *name)
 		return;
 	}
 
-	toolbar_toplevel_toolbar_create (uih, name, bonobo_object_corba_objref (BONOBO_OBJECT (uih)));
+	toolbar_toplevel_toolbar_create (
+		uih, name, bonobo_object_corba_objref (BONOBO_OBJECT (uih)));
 }
 
 static void
 toolbar_toplevel_toolbar_remove (BonoboUIHandler *uih, const char *name)
 {
-	g_warning ("toolbar remove unimplemented");
+	ToolbarToolbarInternal *internal;
+
+	internal = toolbar_toplevel_get_toolbar (uih, name);
+	g_return_if_fail (internal != NULL);
+
+	/* FIXME: This would appear to leave a load of structures around */
+	toolbar_toplevel_toolbar_remove_widgets_recursive (uih, name);
+	
+/*	bonobo_ui_handler_toolbar_free_tree (internal->item);*/
 }
 
 static void
@@ -836,8 +875,8 @@ toolbar_remote_toolbar_remove (BonoboUIHandler *uih, const char *name)
 	CORBA_exception_init (&ev);
 
 	Bonobo_UIHandler_toolbar_remove (uih->top_level_uih,
-					bonobo_object_corba_objref (BONOBO_OBJECT (uih)),
-					name, &ev);
+					 bonobo_object_corba_objref (BONOBO_OBJECT (uih)),
+					 name, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		bonobo_object_check_env (
 			BONOBO_OBJECT (uih),
@@ -848,14 +887,20 @@ toolbar_remote_toolbar_remove (BonoboUIHandler *uih, const char *name)
 }
 
 void
-impl_Bonobo_UIHandler_toolbar_remove (PortableServer_Servant  servant,
-				      Bonobo_UIHandler        containee,
+impl_Bonobo_UIHandler_toolbar_remove (PortableServer_Servant servant,
+				      Bonobo_UIHandler       containee,
 				      const CORBA_char      *name,
-		     CORBA_Environment     *ev)
+				      CORBA_Environment     *ev)
 {
-/*	BonoboUIHandler *uih = BONOBO_UI_HANDLER (bonobo_object_from_servant (servant));*/
+	BonoboUIHandler *uih = BONOBO_UI_HANDLER (bonobo_object_from_servant (servant));
 
-	g_warning ("toolbar remove unimplemented");
+	if (! bonobo_ui_handler_toplevel_check_toplevel (uih)) {
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+				     ex_Bonobo_UIHandler_NotToplevelHandler,
+				     NULL);
+		return;
+	}
+	toolbar_toplevel_toolbar_remove (uih, name);
 }
 
 /**
@@ -864,7 +909,6 @@ impl_Bonobo_UIHandler_toolbar_remove (PortableServer_Servant  servant,
 void
 bonobo_ui_handler_remove_toolbar (BonoboUIHandler *uih, const char *name)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (name != NULL);
 
@@ -881,10 +925,8 @@ void
 bonobo_ui_handler_set_toolbar (BonoboUIHandler *uih, const char *name,
 			       GtkWidget *toolbar)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (name != NULL);
-	g_return_if_fail (toolbar != NULL);
 	g_return_if_fail (GTK_IS_TOOLBAR (toolbar));
 
 	/* This is a toplevel-only function. */
@@ -907,7 +949,6 @@ bonobo_ui_handler_set_toolbar (BonoboUIHandler *uih, const char *name,
 GList *
 bonobo_ui_handler_get_toolbar_list (BonoboUIHandler *uih)
 {
-	g_return_val_if_fail (uih != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_UI_HANDLER (uih), NULL);
 
 	return NULL;
@@ -988,6 +1029,7 @@ toolbar_local_create_item (BonoboUIHandler *uih, const char *parent_path,
 	internal_cb = g_new0 (ToolbarItemLocalInternal, 1);
 	internal_cb->callback = item->callback;
 	internal_cb->callback_data = item->callback_data;
+	internal_cb->callback_data_destroy_notify = item->callback_data_destroy_notify;
 
 	l = g_hash_table_lookup (uih->path_to_toolbar_callback, item->path);
 	new_list = g_list_prepend (l, internal_cb);
@@ -1534,7 +1576,6 @@ void
 bonobo_ui_handler_toolbar_add_one (BonoboUIHandler *uih, const char *parent_path,
 				  BonoboUIHandlerToolbarItem *item)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (parent_path != NULL);
 	g_return_if_fail (item != NULL);
@@ -1560,7 +1601,6 @@ bonobo_ui_handler_toolbar_add_list (BonoboUIHandler *uih, const char *parent_pat
 {
 	BonoboUIHandlerToolbarItem *curr;
 
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (parent_path != NULL);
 	g_return_if_fail (item != NULL);
@@ -1573,7 +1613,6 @@ void
 bonobo_ui_handler_toolbar_add_tree (BonoboUIHandler *uih, const char *parent_path,
 				   BonoboUIHandlerToolbarItem *item)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (parent_path != NULL);
 	g_return_if_fail (item != NULL);
@@ -1598,13 +1637,12 @@ bonobo_ui_handler_toolbar_new (BonoboUIHandler *uih, const char *path,
 			      BonoboUIHandlerPixmapType pixmap_type,
 			      gpointer pixmap_data, guint accelerator_key,
 			      GdkModifierType ac_mods,
-			      BonoboUIHandlerCallbackFunc callback,
+			      BonoboUIHandlerCallback callback,
 			      gpointer callback_data)
 {
 	BonoboUIHandlerToolbarItem *item;
 	char *parent_path;
 
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (path != NULL);
 
@@ -1629,7 +1667,7 @@ bonobo_ui_handler_toolbar_new_item (BonoboUIHandler *uih, const char *path,
 				   BonoboUIHandlerPixmapType pixmap_type,
 				   gpointer pixmap_data,
 				   guint accelerator_key, GdkModifierType ac_mods,
-				   BonoboUIHandlerCallbackFunc callback,
+				   BonoboUIHandlerCallback callback,
 				   gpointer callback_data)
 {
 	bonobo_ui_handler_toolbar_new (uih, path,
@@ -1688,7 +1726,7 @@ bonobo_ui_handler_toolbar_new_radioitem (BonoboUIHandler *uih, const char *path,
 					BonoboUIHandlerPixmapType pixmap_type,
 					gpointer pixmap_data,
 					guint accelerator_key, GdkModifierType ac_mods,
-					BonoboUIHandlerCallbackFunc callback,
+					BonoboUIHandlerCallback callback,
 					gpointer callback_data)
 {
 	bonobo_ui_handler_toolbar_new (uih, path, BONOBO_UI_HANDLER_TOOLBAR_RADIOITEM,
@@ -1706,7 +1744,7 @@ bonobo_ui_handler_toolbar_new_toggleitem (BonoboUIHandler *uih, const char *path
 					 BonoboUIHandlerPixmapType pixmap_type,
 					 gpointer pixmap_data,
 					 guint accelerator_key, GdkModifierType ac_mods,
-					 BonoboUIHandlerCallbackFunc callback,
+					 BonoboUIHandlerCallback callback,
 					 gpointer callback_data)
 {
 	bonobo_ui_handler_toolbar_new (uih, path, BONOBO_UI_HANDLER_TOOLBAR_TOGGLEITEM,
@@ -1987,7 +2025,6 @@ impl_Bonobo_UIHandler_toolbar_remove_item (PortableServer_Servant  servant,
 void
 bonobo_ui_handler_toolbar_remove (BonoboUIHandler *uih, const char *path)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (path != NULL);
 
@@ -2000,7 +2037,6 @@ bonobo_ui_handler_toolbar_remove (BonoboUIHandler *uih, const char *path)
 BonoboUIHandlerToolbarItem *
 bonobo_ui_handler_toolbar_fetch_one (BonoboUIHandler *uih, const char *path)
 {
-	g_return_val_if_fail (uih != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_UI_HANDLER (uih), NULL);
 	g_return_val_if_fail (path != NULL, NULL);
 
@@ -2010,9 +2046,8 @@ bonobo_ui_handler_toolbar_fetch_one (BonoboUIHandler *uih, const char *path)
 
 GList *
 bonobo_ui_handler_toolbar_fetch_by_callback (BonoboUIHandler *uih,
-					    BonoboUIHandlerCallbackFunc callback)
+					    BonoboUIHandlerCallback callback)
 {
-	g_return_val_if_fail (uih != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_UI_HANDLER (uih), NULL);
 
 	g_warning ("Unimplemented toolbar method");
@@ -2023,7 +2058,6 @@ GList *
 bonobo_ui_handler_toolbar_fetch_by_callback_data (BonoboUIHandler *uih,
 						 gpointer callback_data)
 {
-	g_return_val_if_fail (uih != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_UI_HANDLER (uih), NULL);
 
 	g_warning ("Unimplemented toolbar method");
@@ -2325,6 +2359,18 @@ toolbar_item_remote_get_pos (BonoboUIHandler *uih, const char *path)
 	return ans;
 }
 
+int
+bonobo_ui_handler_toolbar_item_get_pos (BonoboUIHandler *uih, const char *path)
+{
+	g_return_val_if_fail (path != NULL, -1);
+	g_return_val_if_fail (BONOBO_IS_UI_HANDLER (uih), -1);
+
+	if (uih->top_level_uih != CORBA_OBJECT_NIL)
+		return toolbar_item_remote_get_pos (uih, path);
+
+	return toolbar_toplevel_item_get_pos (uih, path);
+}
+
 static void
 toolbar_toplevel_item_set_sensitivity (BonoboUIHandler *uih, const char *path, gboolean sensitive)
 {
@@ -2371,9 +2417,8 @@ void
 bonobo_ui_handler_toolbar_item_set_sensitivity (BonoboUIHandler *uih, const char *path,
 					       gboolean sensitive)
 {
-	g_return_if_fail (uih != NULL);
-	g_return_if_fail (path != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
+	g_return_if_fail (path != NULL);
 
 	if (uih->top_level_uih != CORBA_OBJECT_NIL)
 		toolbar_item_remote_set_sensitivity (uih, path, sensitive);
@@ -2427,7 +2472,7 @@ impl_Bonobo_UIHandler_toolbar_item_get_attributes (PortableServer_Servant   serv
 				     NULL);
 		return;
 	}
-	internal = toolbar_toplevel_get_item (uih, path);
+	internal = toolbar_toplevel_get_item_for_containee (uih, path, containee);
 
 	if (internal == NULL) {
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
@@ -2435,20 +2480,13 @@ impl_Bonobo_UIHandler_toolbar_item_get_attributes (PortableServer_Servant   serv
 		return;
 	}
 
-	g_warning ("toolbar_item_get_attributes substantially unimplemented");
-
-	*pos             = toolbar_toplevel_item_get_pos (uih, (char *) path);
-	*sensitive       = toolbar_toplevel_item_get_sensitivity (uih, path);
+	*pos             = internal->item->pos;
+	*sensitive       = internal->sensitive;
 	*toggle_state    = toolbar_toplevel_item_get_toggle_state (uih, path);
-
-	/* FIXME.  The following is just to prevent uninitialized
-           memory accesses.  (And, specifically, to prevent the stub
-           from crashing if it tries to free the strings we are
-           supposed to return.)  */
-	*label           = CORBA_string_dup ("FIXME");
-	*hint            = CORBA_string_dup ("FIXME");
-	*accelerator_key = 0;
-	*ac_mods         = 0;
+	*label           = CORBA_string_dup (internal->item->label);
+	*hint            = CORBA_string_dup (internal->item->hint);
+	*accelerator_key = internal->item->accelerator_key;
+	*ac_mods         = internal->item->ac_mods;
 }
 
 void
@@ -2597,7 +2635,6 @@ void
 bonobo_ui_handler_toolbar_item_set_pixmap (BonoboUIHandler *uih, const char *path,
 					  BonoboUIHandlerPixmapType type, gpointer data)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (path != NULL);
 	g_return_if_fail (data != NULL);
@@ -2614,7 +2651,6 @@ void
 bonobo_ui_handler_toolbar_item_get_pixmap (BonoboUIHandler *uih, const char *path,
 					  BonoboUIHandlerPixmapType *type, gpointer *data)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (path != NULL);
 
@@ -2849,7 +2885,6 @@ bonobo_ui_handler_toolbar_get_orientation (BonoboUIHandler *uih, const char *pat
 gboolean
 bonobo_ui_handler_toolbar_item_get_sensitivity (BonoboUIHandler *uih, const char *path)
 {
-	g_return_val_if_fail (uih != NULL, FALSE);
 	g_return_val_if_fail (BONOBO_IS_UI_HANDLER (uih), FALSE);
 	g_return_val_if_fail (path != NULL, FALSE);
 
@@ -2863,9 +2898,8 @@ void
 bonobo_ui_handler_toolbar_item_set_label (BonoboUIHandler *uih, const char *path,
 					 gchar *label)
 {
-	g_return_if_fail (uih != NULL);
-	g_return_if_fail (path != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
+	g_return_if_fail (path != NULL);
 
 	if (uih->top_level_uih != CORBA_OBJECT_NIL) {
 		toolbar_item_remote_set_label (uih, path, label);
@@ -2878,7 +2912,6 @@ bonobo_ui_handler_toolbar_item_set_label (BonoboUIHandler *uih, const char *path
 gchar *
 bonobo_ui_handler_toolbar_item_get_label (BonoboUIHandler *uih, const char *path)
 {
-	g_return_val_if_fail (uih != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_UI_HANDLER (uih), NULL);
 	g_return_val_if_fail (path != NULL, NULL);
 
@@ -2890,7 +2923,6 @@ void
 bonobo_ui_handler_toolbar_item_get_accel (BonoboUIHandler *uih, const char *path,
 					 guint *accelerator_key, GdkModifierType *ac_mods)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (path != NULL);
 
@@ -2899,10 +2931,10 @@ bonobo_ui_handler_toolbar_item_get_accel (BonoboUIHandler *uih, const char *path
 
 void
 bonobo_ui_handler_toolbar_item_set_callback (BonoboUIHandler *uih, const char *path,
-					    BonoboUIHandlerCallbackFunc callback,
-					    gpointer callback_data)
+					     BonoboUIHandlerCallback callback,
+					     gpointer callback_data,
+					     GDestroyNotify callback_data_destroy_notify)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (path != NULL);
 
@@ -2911,10 +2943,19 @@ bonobo_ui_handler_toolbar_item_set_callback (BonoboUIHandler *uih, const char *p
 
 void
 bonobo_ui_handler_toolbar_item_get_callback (BonoboUIHandler *uih, const char *path,
-					    BonoboUIHandlerCallbackFunc *callback,
-					    gpointer *callback_data)
+					     BonoboUIHandlerCallback *callback,
+					     gpointer *callback_data,
+					     GDestroyNotify *callback_data_destroy_notify)
+{
+	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
+	g_return_if_fail (path != NULL);
+
+	g_warning ("Unimplemented toolbar method");
+}
+
+void
+bonobo_ui_handler_toolbar_item_remove_callback_no_notify (BonoboUIHandler *uih, const char *path)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (path != NULL);
 
@@ -2952,7 +2993,6 @@ void
 bonobo_ui_handler_toolbar_item_toggle_set_state (BonoboUIHandler *uih, const char *path,
 						 gboolean state)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (path != NULL);
 
@@ -2996,7 +3036,6 @@ toolbar_toplevel_item_get_state (BonoboUIHandler *uih, const char *path)
 gboolean
 bonobo_ui_handler_toolbar_item_toggle_get_state (BonoboUIHandler *uih, const char *path)
 {
-	g_return_val_if_fail (uih != NULL,FALSE);
 	g_return_val_if_fail (BONOBO_IS_UI_HANDLER (uih), FALSE);
 	g_return_val_if_fail (path != NULL, FALSE);
 
@@ -3009,7 +3048,6 @@ bonobo_ui_handler_toolbar_item_toggle_get_state (BonoboUIHandler *uih, const cha
 gboolean
 bonobo_ui_handler_toolbar_item_radio_get_state (BonoboUIHandler *uih, const char *path)
 {
-	g_return_val_if_fail (uih != NULL, FALSE);
 	g_return_val_if_fail (BONOBO_IS_UI_HANDLER (uih), FALSE);
 	g_return_val_if_fail (path != NULL, FALSE);
 
@@ -3021,7 +3059,6 @@ void
 bonobo_ui_handler_toolbar_item_radio_set_state (BonoboUIHandler *uih, const char *path,
 					       gboolean state)
 {
-	g_return_if_fail (uih != NULL);
 	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
 	g_return_if_fail (path != NULL);
 
