@@ -34,9 +34,9 @@ static BonoboControlFrameClass *bonobo_view_frame_parent_class;
 POA_Bonobo_ViewFrame__vepv bonobo_view_frame_vepv;
 
 struct _BonoboViewFramePrivate {
-	GtkWidget	*wrapper; 
+	GtkWidget	 *wrapper; 
 	BonoboClientSite *client_site;
-	BonoboUIHandler	*uih;
+	BonoboUIHandler	 *uih;
 	Bonobo_View       view;
 };
 
@@ -130,7 +130,8 @@ bonobo_view_frame_construct (BonoboViewFrame  *view_frame,
 	g_return_val_if_fail (client_site != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_CLIENT_SITE (client_site), NULL);
 
-	bonobo_control_frame_construct (BONOBO_CONTROL_FRAME (view_frame), corba_view_frame, uih);
+	bonobo_control_frame_construct (BONOBO_CONTROL_FRAME (view_frame),
+					corba_view_frame, uih);
 
 	view_frame->priv->client_site = client_site;
 	
@@ -140,7 +141,7 @@ bonobo_view_frame_construct (BonoboViewFrame  *view_frame,
 	 */
 	wrapper = bonobo_wrapper_new ();
 	if (wrapper == NULL) {
-		gtk_object_unref (GTK_OBJECT (view_frame));
+		bonobo_object_unref (BONOBO_OBJECT (view_frame));
 		return NULL;
 	}
 	gtk_object_ref (GTK_OBJECT (wrapper));
@@ -185,7 +186,7 @@ bonobo_view_frame_new (BonoboClientSite *client_site,
 
 	corba_view_frame = create_bonobo_view_frame (BONOBO_OBJECT (view_frame));
 	if (corba_view_frame == CORBA_OBJECT_NIL) {
-		gtk_object_destroy (GTK_OBJECT (view_frame));
+		bonobo_object_unref (BONOBO_OBJECT (view_frame));
 		return NULL;
 	}
 
@@ -197,7 +198,7 @@ bonobo_view_frame_destroy (GtkObject *object)
 {
 	BonoboViewFrame *view_frame = BONOBO_VIEW_FRAME (object);
 
-	if (view_frame->priv->view != CORBA_OBJECT_NIL){
+	if (view_frame->priv->view != CORBA_OBJECT_NIL) {
 		CORBA_Environment ev;
 
 		CORBA_exception_init (&ev);
@@ -206,7 +207,7 @@ bonobo_view_frame_destroy (GtkObject *object)
 		CORBA_exception_free (&ev);
 	}
 	
-	gtk_object_destroy (GTK_OBJECT (view_frame->priv->wrapper));
+	gtk_object_unref (GTK_OBJECT (view_frame->priv->wrapper));
 	g_free (view_frame->priv);
 	
 	GTK_OBJECT_CLASS (bonobo_view_frame_parent_class)->destroy (object);
@@ -319,12 +320,15 @@ bonobo_view_frame_bind_to_view (BonoboViewFrame *view_frame, Bonobo_View view)
 	CORBA_Environment ev;
 
 	g_return_if_fail (view_frame != NULL);
+	g_return_if_fail (view != CORBA_OBJECT_NIL);
 	g_return_if_fail (BONOBO_IS_VIEW_FRAME (view_frame));
 
 	CORBA_exception_init (&ev);
 	bonobo_control_frame_bind_to_control (
 		BONOBO_CONTROL_FRAME (view_frame),
 		(Bonobo_Control) view);
+	
+	Bonobo_Unknown_ref (view, &ev);
 	view_frame->priv->view = CORBA_Object_duplicate (view, &ev);
 
 	CORBA_exception_free (&ev);
