@@ -360,7 +360,7 @@ bonobo_widget_destroy (GtkObject *object)
 		bonobo_object_unref (BONOBO_OBJECT (priv->client_site));
 	if (priv->view_frame)
 		bonobo_object_unref (BONOBO_OBJECT (priv->view_frame));
-	if (priv->uih){
+	if (priv->uih != CORBA_OBJECT_NIL) {
 		CORBA_Environment ev;
 
 		CORBA_exception_init (&ev);
@@ -483,9 +483,9 @@ bonobo_widget_set_property (BonoboWidget      *control,
 	if (err)
 		g_warning ("Error '%s'", err);
 
-	va_end (args);
+	bonobo_object_unref (BONOBO_OBJECT (cl));
 
-	gtk_object_unref (GTK_OBJECT (cl));
+	va_end (args);
 }
 
 void
@@ -508,5 +508,7 @@ bonobo_widget_get_property (BonoboWidget      *control,
 	if ((err = bonobo_property_bag_client_getv (cl, first_prop, args)))
 		g_warning ("Error '%s'", err);
 
+	bonobo_object_unref (BONOBO_OBJECT (cl));
+
 	va_end (args);
 }
