@@ -62,7 +62,7 @@ GtkWidget          *bonobo_widget_new_subdoc               (const char       *ob
 BonoboContainer    *bonobo_widget_get_container            (BonoboWidget     *bw);
 BonoboClientSite   *bonobo_widget_get_client_site          (BonoboWidget     *bw);
 BonoboViewFrame    *bonobo_widget_get_view_frame           (BonoboWidget     *bw);
-BonoboUIHandler    *bonobo_widget_get_uih                  (BonoboWidget     *bw);
+Bonobo_UIHandler    bonobo_widget_get_uih                  (BonoboWidget     *bw);
 
 void                bonobo_widget_set_property            (BonoboWidget      *control,
 							   const char        *first_prop,
