@@ -11,6 +11,10 @@
 #ifndef BONOBO_H
 #define BONOBO_H 1
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include <bonobo/bonobo-object.h>
 
 #include <bonobo/bonobo-moniker.h>
@@ -31,6 +35,7 @@
 
 #include <bonobo/bonobo-embeddable.h>
 #include <bonobo/bonobo-embeddable-factory.h>
+#include <bonobo/bonobo-sizeable.h>
 
 #include <bonobo/bonobo-main.h>
 
@@ -57,4 +62,8 @@
 #include <bonobo/bonobo-print.h>
 #include <bonobo/bonobo-print-client.h>
 
+#ifdef __cplusplus
+}
 #endif
+
+#endif /* BONOBO_H */
