@@ -58,7 +58,7 @@ static void
 embeddable_system_exception_cb (BonoboEmbeddable *embeddable, CORBA_Object corba_object,
 				CORBA_Environment *ev, gpointer data)
 {
-	bonobo_object_destroy (BONOBO_OBJECT (embeddable));
+	bonobo_object_unref (BONOBO_OBJECT (embeddable));
 }
 
 /*
@@ -68,7 +68,7 @@ static void
 view_system_exception_cb (BonoboView *view, CORBA_Object corba_object,
 			  CORBA_Environment *ev, gpointer data)
 {
-	bonobo_object_destroy (BONOBO_OBJECT (view));
+	bonobo_object_unref (BONOBO_OBJECT (view));
 }
 
 static void
@@ -104,6 +104,7 @@ load_board (BonoboPersistStream        *ps,
 	embeddable_data_t    *embed = closure;
 	MineField            *mf;
 	Bonobo_Stream_iobuf  *buffer;
+	char                 *str;
 	int                   bx, by, j;
 
 	g_return_if_fail (embed != NULL);
@@ -117,16 +118,14 @@ load_board (BonoboPersistStream        *ps,
 
 	mf = embed->data;
 
-	Bonobo_Stream_read (stream, 5, &buffer, ev);
-	if (buffer->_length != 5) {
+	bonobo_stream_client_read_string (stream, &str, ev);
+	if (ev->_major != CORBA_NO_EXCEPTION || str == NULL) {
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
 				     ex_Bonobo_Persist_WrongDataType, NULL);
 		return;
 	}
-
-	sscanf (buffer->_buffer, "%2u%2u\n", &bx, &by);
-
-	CORBA_free (buffer);
+	sscanf (str, "%2u%2u\n", &bx, &by);
+	g_free (str);
 
 	if (bx > 128 || by > 128) {
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
@@ -140,8 +139,10 @@ load_board (BonoboPersistStream        *ps,
 	for (j = 0; j < by; j++) {
 		int i;
 
-		Bonobo_Stream_read (stream, bx*2 + 1, &buffer, ev);
-		if (buffer->_length != bx + 1) {
+		Bonobo_Stream_read (stream, bx * 2 + 1, &buffer, ev);
+		if (ev->_major != CORBA_NO_EXCEPTION)
+			return;
+		else if (buffer->_length != bx * 2 + 1) {
 			CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
 					     ex_Bonobo_Persist_WrongDataType,
 					     NULL);
@@ -149,8 +150,8 @@ load_board (BonoboPersistStream        *ps,
 		}
 
 		for (i = 0; i < bx; i++)
-			minefield_set_at (mf, i, j, buffer->_buffer[(i << 1)],
-					  buffer->_buffer[(i << 1) + 1]);
+			minefield_set_at (mf, i, j, buffer->_buffer [(i << 1)],
+					  buffer->_buffer [(i << 1) + 1]);
 
 		CORBA_free (buffer);
 	}
@@ -167,7 +168,6 @@ save_board (BonoboPersistStream        *ps,
 {
 	embeddable_data_t    *embed = closure;
 	MineField            *mf;
-	Bonobo_Stream_iobuf  *buffer;
 	char                 *data;
 	int                   j;
 
@@ -182,32 +182,26 @@ save_board (BonoboPersistStream        *ps,
 
 	mf = embed->data;
 
-	data = g_strdup_printf ("%2u%2u\n", mf->xsize, mf->ysize);
-	buffer = Bonobo_Stream_iobuf__alloc ();
-    	buffer->_buffer = data;
-	buffer->_length = strlen (data);
-	Bonobo_Stream_write (stream, buffer, ev);
-	CORBA_free (buffer);
+	bonobo_stream_client_printf (stream, TRUE, ev, "%2u%2u\n",
+				     mf->xsize, mf->ysize);
 	if (ev->_major != CORBA_NO_EXCEPTION)
 		return;
 
+	data = g_malloc (mf->xsize * 2 + 1);
 	for (j = 0; j < mf->ysize; j++) {
 		int i;
 
-		buffer = Bonobo_Stream_iobuf__alloc ();
-		buffer->_length = mf->xsize*2 + 1;
-		buffer->_buffer = g_malloc (buffer->_length);
-
 		for (i = 0; i < mf->xsize; i++)
-			minefield_get_at (mf, i, j, &buffer->_buffer[(i << 1)],
-					  &buffer->_buffer[(i << 1) + 1]);
-		buffer->_buffer[(i << 1)] = '\n';
+			minefield_get_at (mf, i, j, &data [(i << 1)],
+					  &data [(i << 1) + 1]);
+		data [(i << 1)] = '\n';
+
+		bonobo_stream_client_write (stream, data, mf->xsize * 2 + 1, ev);
 
-		Bonobo_Stream_write (stream, buffer, ev);
-		CORBA_free (buffer);
 		if (ev->_major != CORBA_NO_EXCEPTION)
 			return;
 	}
+	g_free (data);
 }
 
 static Bonobo_Persist_ContentTypeList *
@@ -225,7 +219,9 @@ view_destroy_cb (BonoboView *view, gpointer data)
 {
 	view_data_t *view_data = (view_data_t *) data;
 
-	gtk_widget_destroy (GTK_WIDGET (view_data->mfield));
+	view_data->view = NULL;
+	view_data->mfield = NULL;
+
 	g_free (view_data);
 }
 
@@ -245,6 +241,13 @@ view_new_game_ui_cb  (BonoboUIHandler *uih, void *user_data,
 	embeddable_update_all_views (view_data->embeddable_data);
 }
 
+static void
+load_game_ui_cb (BonoboUIHandler *uih, void *user_data,
+		 const char *path)
+{
+	g_warning ("As yet, unimplemented; need to create a stream");
+}
+
 static void
 view_new_game_cb (BonoboView *view, const char *verb_name, void *user_data)
 {
@@ -293,6 +296,20 @@ view_create_menus (view_data_t *view_data)
 	/*
 	 * Create our menu entries.
 	 */
+	if (bonobo_ui_handler_menu_path_exists (uih, "/File/Placeholder")) {
+		bonobo_ui_handler_menu_new_item (uih,  "/File/Placeholder/NewGame",
+						 L_("_New game"), _("Start a new game"), -1,
+						 BONOBO_UI_HANDLER_PIXMAP_NONE, NULL, 0,
+						 (GdkModifierType)0, view_new_game_ui_cb,
+						 (gpointer)view_data);
+		bonobo_ui_handler_menu_new_item (uih,  "/File/Placeholder/OpenGame",
+						 L_("_Open game"), _("Load a saved game"), -1,
+						 BONOBO_UI_HANDLER_PIXMAP_NONE, NULL, 0,
+						 (GdkModifierType)0, load_game_ui_cb,
+						 (gpointer)view_data);		
+	} else
+		g_warning ("No File placeholder");
+
 	bonobo_ui_handler_menu_new_subtree (uih, "/Game",
 					   L_("_Game"),
 					   _("Change the game"),
@@ -303,7 +320,8 @@ view_create_menus (view_data_t *view_data)
 	bonobo_ui_handler_menu_new_item    (uih,  "/Game/NewGame",
 					   L_("_New game"), _("Start a new game"), -1,
 					   BONOBO_UI_HANDLER_PIXMAP_NONE, NULL, 0,
-					   (GdkModifierType)0, view_new_game_ui_cb, (gpointer)view_data);
+					   (GdkModifierType)0, view_new_game_ui_cb,
+					    (gpointer)view_data);
 
 	/* Add out new game toolbar */
 	bonobo_ui_handler_create_toolbar (uih, "Gnomines");
@@ -561,7 +579,7 @@ embeddable_factory (BonoboEmbeddableFactory *this,
 					    embeddable_data);
 
 	if (stream == NULL) {
-		gtk_object_unref (GTK_OBJECT (embeddable));
+		bonobo_object_unref (BONOBO_OBJECT (embeddable));
 		g_free (embeddable_data);
 		return NULL;
 	}
