@@ -276,7 +276,7 @@ view_system_exception_cb (BonoboView *view, CORBA_Object corba_object,
 			  CORBA_Environment *ev, gpointer data)
 {
 	g_warning ("View system exception");
-	bonobo_object_destroy (BONOBO_OBJECT (view));
+	bonobo_object_unref (BONOBO_OBJECT (view));
 }
 
 static void
@@ -284,7 +284,7 @@ embeddable_system_exception_cb (BonoboEmbeddable *embeddable, CORBA_Object corba
 				CORBA_Environment *ev, gpointer data)
 {
 	g_warning ("Embeddable system exception");
-	bonobo_object_destroy (BONOBO_OBJECT (embeddable));
+	bonobo_object_unref (BONOBO_OBJECT (embeddable));
 }
 
 static void
@@ -508,19 +508,17 @@ stream_read (Bonobo_Stream stream, bonobo_object_data_t *bonobo_object_data,
 	     CORBA_Environment *ev)
 {
 	Bonobo_Stream_iobuf *buffer;
-	CORBA_long bytes_read;
 
 	do {
-
 #define READ_CHUNK_SIZE 65536
-		bytes_read = Bonobo_Stream_read (stream, READ_CHUNK_SIZE,
-						&buffer, ev);
+		Bonobo_Stream_read (stream, READ_CHUNK_SIZE,
+				    &buffer, ev);
 		if (ev->_major != CORBA_NO_EXCEPTION)
 			return;
 
 		bonobo_object_data->text = g_realloc (bonobo_object_data->text,
-						      bonobo_object_data->text_len
-						      + buffer->_length);
+						      bonobo_object_data->text_len +
+						      buffer->_length);
 
 		memcpy (bonobo_object_data->text
 			+ bonobo_object_data->text_len,
@@ -528,8 +526,13 @@ stream_read (Bonobo_Stream stream, bonobo_object_data_t *bonobo_object_data,
 
 		bonobo_object_data->text_len += buffer->_length;
 
+		if (buffer->_length <= 0)
+			break;
+
 		CORBA_free (buffer);
-	} while (bytes_read > 0);
+	} while (1);
+
+	CORBA_free (buffer);
 } /* stream_read */
 
 /*
@@ -542,7 +545,7 @@ pstream_load (BonoboPersistStream *ps, const Bonobo_Stream stream,
 {
 	bonobo_object_data_t *bonobo_object_data = data;
 
-	if (*type && g_strcasecmp (type, "text/plain") != 0) {	    
+	if (*type && g_strncasecmp (type, "text/", 5) != 0) {	    
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
 				     ex_Bonobo_Persist_WrongDataType, NULL);
 		return;
@@ -858,8 +861,8 @@ embeddable_factory (BonoboEmbeddableFactory *this,
 					    pstream_get_max_size,
 					    pstream_get_content_types,
 					    bonobo_object_data);
-	if (stream == NULL) {
-		gtk_object_unref (GTK_OBJECT (bonobo_object));
+	if (!stream) {
+		bonobo_object_unref (BONOBO_OBJECT (bonobo_object));
 		g_free (bonobo_object_data);
 		return NULL;
 	}
@@ -876,8 +879,8 @@ embeddable_factory (BonoboEmbeddableFactory *this,
 						 progressive_add_data,
 						 progressive_set_size,
 						 bonobo_object_data);
-	if (psink == NULL) {
-		gtk_object_unref (GTK_OBJECT (bonobo_object));
+	if (!psink) {
+		bonobo_object_unref (BONOBO_OBJECT (bonobo_object));
 		g_free (bonobo_object_data);
 		return NULL;
 	}
