@@ -131,9 +131,11 @@ efs_open (const char *path, gint flags, gint mode)
 		if (drivername[0] && (!(d = efs_find_driver(drivername)))) {
 			g_warning ("Unable to find driver!");
 		}	
-		if (!d) d = efs_driver_list[0];  /* default driver */  
 	}
 
+	if (!d)
+		d = efs_driver_list [0];  /* default driver */
+
 	efs = g_malloc0(sizeof(EFS));
 	efs->driver = d;
 	return d->sops->open (efs, filename, flags, mode);
