@@ -48,7 +48,7 @@ EFSDriver efs_driver_simple = {
 	&file_ops_comp_simple
 };
 
-guint32
+static guint32
 simple_calc_csum (SimpleHeader *head)
 {
 	guint32 csum = adler32 (0L, NULL, 0);
@@ -142,7 +142,7 @@ simple_open (EFS *efs, const char *path, gint flags, gint mode)
 	gint i,j,fbc,pflags;
 	guint32 inode,k,*buf;
 	SimpleDir *dir;
-	// struct flock fl = {F_WRLCK, SEEK_SET, 0, 0};
+	/* struct flock fl = {F_WRLCK, SEEK_SET, 0, 0}; */
 
 	simple_init (efs);
 	sefs = (SimpleEFS *)efs->pdata;
@@ -514,7 +514,7 @@ static gint
 simple_close(EFS *efs)
 {
 	SimpleEFS *sefs = (SimpleEFS *)efs->pdata;
-	// struct flock fl = {F_UNLCK, SEEK_SET, 0, 0};
+	/* struct flock fl = {F_UNLCK, SEEK_SET, 0, 0}; */
 	gint i;
 
 	if (efs->mode&EFS_WRITE) {
