@@ -1,4 +1,4 @@
-/* $Id: bonobo-hello.c,v 1.4 2000/05/23 16:43:08 cactus Exp $ */
+/* $Id: bonobo-hello.c,v 1.9 2000/07/14 18:10:50 mmeeks Exp $ */
 /*
   Bonobo-Hello Copyright (C) 2000 RDI Gerg <cactus@cactus.rulez.org>
   
@@ -26,42 +26,88 @@
 #endif
 #include <bonobo.h>
 
-#include "hello-object.h"
+#include "hello-embeddable.h"
+
+static BonoboGenericFactory *factory = NULL;
+static gint running_objects = 0;
 
 static void
-init_bonobo_hello_factory (void)
+hello_bonobo_destroy (BonoboEmbeddable *embeddable,
+		      gpointer user_data)
 {
-#if USING_OAF
-	factory =
-	    bonobo_embeddable_factory_new ("OAFIID:bonobo-hello-factory:cactus",
-					   hello_object_factory,
-					   NULL);
-#else
-	factory =
-	    bonobo_embeddable_factory_new ("bonobo-object-factory:hello",
-					   hello_object_factory, NULL);
-#endif
+	running_objects--;
+	if (running_objects > 0)
+		return;
+
+	if (factory)
+		bonobo_object_unref (BONOBO_OBJECT (factory));
+	else
+		g_warning ("Serious ref counting error");
+	factory = NULL;
+
+	gtk_main_quit ();
+}
+
+static BonoboObject*
+hello_embeddable_factory (BonoboEmbeddableFactory *f, gpointer data)
+{
+	HelloBonoboEmbeddable *embeddable;
+
+	embeddable = gtk_type_new (HELLO_BONOBO_EMBEDDABLE_TYPE);
+
+	g_return_val_if_fail(embeddable != NULL, NULL);
+
+	running_objects++;
+
+	/* Install destructor */
+	gtk_signal_connect (GTK_OBJECT(embeddable), "destroy",
+			    GTK_SIGNAL_FUNC(hello_bonobo_destroy),
+			    NULL);
+
+	embeddable = hello_bonobo_embeddable_construct (embeddable);
+
+	return BONOBO_OBJECT (embeddable);
 }
 
 static void
-init_server_factory (int argc, char **argv)
+hello_bonobo_init (void)
+{
+#if USING_OAF
+	factory =
+		bonobo_embeddable_factory_new (
+			"OAFIID:bonobo-hello-factory:413433d0-c643-4618-9c3e-2c99f4d1b2a0",
+			hello_embeddable_factory, NULL);
+#else
+	factory =
+		bonobo_embeddable_factory_new ("bonobo-object-factory:hello",
+					       hello_embeddable_factory, NULL);
+#endif
+	if (!factory)
+		g_warning ("Couldn't register hello object factory");
+}
+
+static void
+server_factory_init (int argc, char **argv)
 {
 	CORBA_Environment ev;
+	CORBA_ORB orb;
+
 	CORBA_exception_init (&ev);
 
 #if USING_OAF
 	gnome_init_with_popt_table (PACKAGE, VERSION,
 				    argc, argv, oaf_popt_options, 0, NULL);
-	oaf_init (argc, argv);
+	orb = oaf_init (argc, argv);
 #else
 	gnome_CORBA_init_with_popt_table (PACKAGE, VERSION,	/* Name of the component */
 					  &argc, argv,	/* Command-line arguments */
 					  NULL, 0, NULL,	/* popt table to parse arguments */
 					  GNORBA_INIT_SERVER_FUNC, &ev	/* GNORBA options */
 	    );
+	orb = gnome_CORBA_ORB ();
 #endif
 
-	if (!bonobo_init (CORBA_OBJECT_NIL,
+	if (!bonobo_init (orb,
 			  CORBA_OBJECT_NIL,
 			  CORBA_OBJECT_NIL))
 		    g_error (_("I could not initialize Bonobo"));
@@ -72,8 +118,8 @@ init_server_factory (int argc, char **argv)
 int
 main (int argc, char **argv)
 {
-	init_server_factory (argc, argv);
-	init_bonobo_hello_factory ();
+	server_factory_init (argc, argv);
+	hello_bonobo_init ();
 
 	bonobo_main ();
 
