@@ -104,7 +104,7 @@ static void
 embeddable_system_exception_cb (BonoboEmbeddable *embeddable, CORBA_Object corba_object,
 				CORBA_Environment *ev, gpointer data)
 {
-	bonobo_object_destroy (BONOBO_OBJECT (embeddable));
+	bonobo_object_unref (BONOBO_OBJECT (embeddable));
 }
 
 /*
@@ -114,7 +114,7 @@ static void
 view_system_exception_cb (BonoboView *view, CORBA_Object corba_object,
 			  CORBA_Environment *ev, gpointer data)
 {
-	bonobo_object_destroy (BONOBO_OBJECT (view));
+	bonobo_object_unref (BONOBO_OBJECT (view));
 }
 
 /*
@@ -194,6 +194,7 @@ view_create_menus (view_data_t *view_data)
 	Bonobo_UIHandler  remote_uih;
 	BonoboView       *view = view_data->view;
 	BonoboUIHandler  *uih;
+	int               i;
 
 	/*
 	 * Grab our BonoboUIHandler object.
@@ -251,6 +252,10 @@ view_create_menus (view_data_t *view_data)
 					     0, (GdkModifierType) 0,
 					     GTK_SIGNAL_FUNC (view_color_select_cb), (gpointer) view_data);
 
+	g_warning ("Breakpoint");
+	i = bonobo_ui_handler_menu_get_pos (uih, "/Colors/color radiogroup/Green");
+
+	i = bonobo_ui_handler_menu_get_pos (uih, "/Colors");
 }
 
 /*
@@ -737,7 +742,6 @@ init_server_factory (int argc, char **argv)
 	orb = gnome_CORBA_ORB ();
 #endif
 
-
 	if (bonobo_init (orb, NULL, NULL) == FALSE)
 		g_error (_("Could not initialize Bonobo!"));
 }
