@@ -28,25 +28,11 @@
 static BonoboStorage *
 create_storage_efs_server (const BonoboStorageEFS *storage_efs)
 {
-	BonoboObject *object = BONOBO_OBJECT(storage_efs);
-	POA_Bonobo_Storage *servant;
 	Bonobo_Storage corba_storage;
-	CORBA_Environment ev;
 
-	servant = (POA_Bonobo_Storage *) g_new0 (BonoboObjectServant, 1);
-	servant->vepv = &bonobo_storage_vepv;
-
-	CORBA_exception_init (&ev);
-	POA_Bonobo_Storage__init ((PortableServer_Servant) servant, 
-				 &ev);
-	if (ev._major != CORBA_NO_EXCEPTION){
-		g_free (servant);
-		CORBA_exception_free (&ev);
-		return NULL;
-	}
-	CORBA_exception_free (&ev);
-
-	corba_storage = bonobo_object_activate_servant(object, servant);
+	corba_storage = bonobo_storage_corba_object_create (
+		BONOBO_OBJECT (storage_efs));
+				 
 	return bonobo_storage_construct(BONOBO_STORAGE(storage_efs), 
 				       corba_storage);
 }
@@ -59,7 +45,7 @@ bonobo_storage_efs_destroy (GtkObject *object)
 	
 	if (storage_efs->owner) {
 		if (storage_efs->dir) efs_dir_close(storage_efs->dir);
-		gtk_object_unref (GTK_OBJECT (storage_efs->owner));    
+		bonobo_object_unref (BONOBO_OBJECT (storage_efs->owner));    
 	} else {
 		if (storage_efs->dir) efs_close (storage_efs->dir);
 	}
@@ -102,7 +88,7 @@ real_create_storage (BonoboStorage *storage, const CORBA_char *path,
 	sefs = gtk_type_new (bonobo_storage_efs_get_type ());
 	sefs->dir = dir;
 	sefs->owner = storage;
-	gtk_object_ref(GTK_OBJECT(storage));
+	bonobo_object_ref (BONOBO_OBJECT(storage));
 	create_storage_efs_server(sefs);
 
 	return BONOBO_STORAGE(sefs);
@@ -125,7 +111,7 @@ real_open_storage (BonoboStorage *storage, const CORBA_char *path,
 	sefs = gtk_type_new (bonobo_storage_efs_get_type ());
 	sefs->dir = dir;
 	sefs->owner = storage;
-	gtk_object_ref(GTK_OBJECT(storage));
+	bonobo_object_ref (BONOBO_OBJECT(storage));
 	create_storage_efs_server(sefs);
 
 	return BONOBO_STORAGE(sefs);
