@@ -12,24 +12,11 @@
 static BonoboStream *
 create_stream_efs_server (const BonoboStreamEFS *stream_efs)
 {
-	BonoboObject *object = BONOBO_OBJECT(stream_efs);
-	POA_Bonobo_Stream *servant;
 	Bonobo_Stream corba_stream;
-	CORBA_Environment ev;
 
-	servant = (POA_Bonobo_Stream *) g_new0 (BonoboObjectServant, 1);
-	servant->vepv = &bonobo_stream_vepv;
+	corba_stream = bonobo_stream_corba_object_create (
+		BONOBO_OBJECT (stream_efs));
 
-	CORBA_exception_init (&ev);
-	POA_Bonobo_Stream__init ((PortableServer_Servant) servant, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION){
-		g_free (servant);
-		CORBA_exception_free (&ev);
-		return NULL;
-	}
-	CORBA_exception_free (&ev);
-
-	corba_stream = bonobo_object_activate_servant(object, servant);
 	return BONOBO_STREAM(bonobo_object_construct(BONOBO_OBJECT(stream_efs), 
 						   corba_stream));
 }
@@ -39,9 +26,10 @@ bonobo_stream_efs_destroy (GtkObject *object)
 {
 	BonoboStreamEFS *stream_efs = BONOBO_STREAM_EFS (object);
 
-	if (stream_efs->file) efs_file_close (stream_efs->file);
+	if (stream_efs->file)
+		efs_file_close (stream_efs->file);
 	if (stream_efs->storage) 
-		gtk_object_unref (GTK_OBJECT (stream_efs->storage));    
+		bonobo_object_unref (BONOBO_OBJECT (stream_efs->storage));    
 }
 
 static CORBA_long
@@ -56,7 +44,7 @@ real_write (BonoboStream *stream, const Bonobo_Stream_iobuf *buffer,
 	return buffer->_length;
 }
 
-static CORBA_long
+static void
 real_read (BonoboStream *stream, CORBA_long count,
 	   Bonobo_Stream_iobuf ** buffer,
 	   CORBA_Environment *ev)
@@ -71,10 +59,16 @@ real_read (BonoboStream *stream, CORBA_long count,
 
 	bytes = efs_file_read(stream_efs->file, data, count);
 
-	(*buffer)->_buffer = data;
-	(*buffer)->_length = bytes;
-
-	return bytes;
+	if (bytes < 0) {
+		CORBA_free (data);
+		CORBA_free (*buffer);
+		*buffer = NULL;
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+				     ex_Bonobo_Stream_IOError, NULL);
+	} else {
+		(*buffer)->_buffer = data;
+		(*buffer)->_length = bytes;
+	}
 }
 
 static CORBA_long
@@ -90,7 +84,6 @@ real_seek (BonoboStream *stream, CORBA_long offset, Bonobo_Stream_SeekType whenc
 		fw = SEEK_END;
 	else
 		fw = SEEK_SET;
-
 	
 	return efs_file_seek (stream_efs->file, offset, fw);
 }
@@ -101,9 +94,16 @@ real_truncate (BonoboStream *stream, const CORBA_long new_size,
 {
 	BonoboStreamEFS *stream_efs = BONOBO_STREAM_EFS (stream);
 
-	if (efs_file_trunc (stream_efs->file, new_size)) {
-		g_warning ("Signal exception!");
-	}
+	if (efs_file_trunc (stream_efs->file, new_size) < 0)
+		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+				     ex_Bonobo_Stream_NoPermission, NULL);
+}
+
+static void
+real_close (BonoboStream      *stream,
+	    CORBA_Environment *ev)
+{
+	g_warning ("FIXME: implement close");
 }
 
 static void
@@ -116,6 +116,7 @@ bonobo_stream_efs_class_init (BonoboStreamEFSClass *class)
 	sclass->read     = real_read;
 	sclass->seek     = real_seek;
 	sclass->truncate = real_truncate;
+        sclass->close    = real_close;
 
 	object_class->destroy = bonobo_stream_efs_destroy;
 }
@@ -163,13 +164,13 @@ bonobo_stream_efs_open_create (BonoboStorageEFS *storage_efs,
 	if (create) flags |= EFS_CREATE;
   
 	if (!(stream_efs->file=efs_file_open(storage_efs->dir, path, flags))) {
-		gtk_object_destroy (GTK_OBJECT (stream_efs));
+		bonobo_object_unref (BONOBO_OBJECT (stream_efs));
 		return NULL;
 	}
 	
 	stream_efs->storage = storage_efs;
-	gtk_object_ref(GTK_OBJECT(storage_efs));
-	create_stream_efs_server(stream_efs);
+	bonobo_object_ref (BONOBO_OBJECT(storage_efs));
+	create_stream_efs_server (stream_efs);
 
 	return BONOBO_STREAM (stream_efs);
 }
